////////////////////////////////////////////////////////////
//            SyntaxElementMorph.prototype.labelPart
// Modified for
// 1. Adding support for the lockedReceivedMessage blocks
// 2. Not allowing varibles to be localized
// 3. Removing the $$ from the label of hidden custom blocks that are visible in student scenarios
////////////////////////////////////////////////////////////
originalSyntaxElementMorphLabelPart = SyntaxElementMorph.prototype.labelPart;
SyntaxElementMorph.prototype.labelPart = function (spec) {
	var part; 
	switch (spec) {
		case '%ls':					//--@@--   for lockedReceiveMessage block
			part = new InputSlotMorph();		
            if (world.role ==='STUDENT') {
				part.notLocal = false ;
				part.locked = true;
				part.isReadOnly = true;
				part.isStatic = true;
                                part.mouseClickLeft = nop;
			} 			
	    break;
		case '%var':
            part = new InputSlotMorph(
                null,
                false,
                'getVarNamesDict',
                true
            );
            part.isStatic = true;
			part.notLocal = true; //do not localize variables
            break;
		case '%shd':
            part = new InputSlotMorph(
                null,
                false,
                'shadowedVariablesMenu',
                true
            );
            part.isStatic = true;
			part.notLocal = true;   //do not localize variables
            break;	
        case '%usr': // network users menu
            part = new InputSlotMorph(
                null,
                false,
                'usersMenu',
                true
            );
            break;
		case '%nusr': // network users menu, including all
            part = new InputSlotMorph(
                null,
                false,
                function(){
					return InputSlotMorph.prototype.usersMenu({"all":["all",""]});
				},
                true
            );
            break;			
		case '%nvar': // network variables menu
            part = new InputSlotMorph(
                null,
                false,
                'nVarsMenu',
                true
            );
            break;	
		case '%svar': // shared variables menu
            part = new InputSlotMorph(
                null,
                false,
                'sharedVarsMenu',
                true
            );
            break;		
		default:
			part = originalSyntaxElementMorphLabelPart.call(this,spec);
	}
	
	return part;
}

// ----- setDraggable ---------------
// Allow a BlockMorph not to be draggable. 
// Used in locking scripts
BlockMorph.prototype.setDraggable = function (status){
        var nextBlock;
	this.isDraggable = status;
         
	if (this instanceof ReporterBlockMorph){
		this.isStatic = !status; 
	}
        this.fixLabelColor();

        if (this.nextBlock) {
            nextBlock=this.nextBlock();
            if (nextBlock){
                nextBlock.setDraggable(status);
           }    
        }
     
}

// --- Undraggable blocks have greyed out labels
originalBlockMorphFixLabelColor = BlockMorph.prototype.fixLabelColor;
BlockMorph.prototype.fixLabelColor = function(){
    var clr, myself=this;
    originalBlockMorphFixLabelColor.call(this);
    if (this.isTemplate || this.isPrototype) {return;}    
        if (!this.isDraggable){
            if (this.zebraContrast > 0 && this.category) {
                var clr = SpriteMorph.prototype.blockColor[this.category];
                if (this.color.eq(clr)) {
                    this.setLabelColor(new Color(220, 220, 220),
                    clr.darker(this.labelContrast),
                    MorphicPreferences.isFlat ? null : new Point(-1, -1)
                    );
                } else {
                    this.setLabelColor(new Color(100, 100, 100),
                        clr.lighter(this.zebraContrast)
                        .lighter(this.labelContrast * 2),
                        MorphicPreferences.isFlat ? null : new Point(1, 1)
                    );            
                }
            }
        }
    
}

BlockMorph.prototype.userMenu = function () {
    var menu = new MenuMorph(this),
        world = this.world(),
        myself = this,
		ide = this.parentThatIsA(IDE_Morph), //++++++++++++++++
		sprite,// = ide.currentSprite,     //+++++++++++++ for depository
                frame,
	shiftClicked = world.currentKey === 16,
        proc = this.activeProcess(),
        vNames = proc && proc.context && proc.context.outerContext ?
                proc.context.outerContext.variables.names() : [],
        //proc ? proc.context.outerContext.variables.names() : [],
        alternatives,
        field,
        rcvr,        
        top,
	blck, blockEditor=null, hintsEditor=null,
        hasNestedUndraggables = (this instanceof CommandBlockMorph && this.hasNestedUndraggables());

// v 4.0.10.2        
        function addOption(label, toggle, test, onHint, offHint) {
        var on = '\u2611 ',
            off = '\u2610 ';
        menu.addItem(
            (test ? on : off) + localize(label),
            toggle,
            test ? onHint : offHint
        );
    }

    function renameVar() {
        var blck = myself.fullCopy();
        blck.addShadow();
        new DialogBoxMorph(
            myself,
            myself.userSetSpec,
            myself
        ).prompt(
            "Variable name",
            myself.blockSpec,
            world,
            blck.fullImage(), // pic
            InputSlotMorph.prototype.getVarNamesDict.call(myself)
        );
    }
// --- v 4.0.10.2

	if (!ide) {
		blockEditor = this.parentThatIsA(BlockEditorMorph);   //restore usermenu in blockeditor 
		if (blockEditor) {
			ide = blockEditor.target.parentThatIsA(IDE_Morph);
		}
                hintsEditor = this.parentThatIsA(HintsEditorMorph);   //restore usermenu in hintseditor 
		if (hintsEditor) {
			ide = hintsEditor.target.parentThatIsA(IDE_Morph);
		}
	}
	

		menu.addItem(
			"help...",
			'showHelp'
		);
	if (!world.HOCmode) {			
		if (shiftClicked) {
			top = this.topBlock();
			if (top instanceof ReporterBlockMorph) {
				menu.addItem(
					"script pic with result...",
					function () {
						top.exportResultPic();
					},
					'open a new window\n' +
						'with a picture of both\nthis script and its result',
					new Color(100, 0, 0)
				);
			}
		}
	}	

	if (this.isTemplate) {
//+++++++++++++++++++++++++++++++++++++++++++++++++++
		if (ide.currentCategory=='depository' && world.role != 'STUDENT'){
			menu.addItem(
						   "delete from depository",
							'deleteFromDepository'
						);
			return menu;
		}
//+++++++++++++++++++++++++++++++++++++++++++++++++++
		if (world.role !== "STUDENT") {
			if (!(this.parent instanceof SyntaxElementMorph)) {
				if (this.selector !== 'reportGetVar' && this.selector !== 'evaluateCustomBlock' && !SpriteMorph.prototype.blocks[this.selector].teachersOnly ) {  //----- if in palette and not custom block or custom variable or teachers only block
					if (this.isHidden) {      // hide-show primitive
						menu.addItem(
						   "show",
							'showPrimitive'
						);
					} else {
							menu.addItem(
							   "hide",
								'hidePrimitive'
							);
					}
// +++++++++++ put primitive block in repository ++++++++++++++ 
					if (!this.isHidden) {//--++ do not put hidden primitives in depository
						menu.addItem(
							"put primitive on depository",
							'putOnDepository'
						);
					} //--++
				}                        
//----------------------------
//====================================== hide Custom variables =============================

				if (this.selector == 'reportGetVar') {  //----- if is custom variable
////// +++++++++++ put only GLOBAL variables in repository ++++++++++++++ ////////////
					sprite = ide.currentSprite;
					frame = sprite.variables.parentFrame.silentFind(this.blockSpec);				
					if (frame && !frame.hiddenVars[this.blockSpec]) {	//--++ do not put hidden vars in depository
						menu.addItem(
						   "put global variable on depository",
						   'putOnDepository'
						);
						addPreference(   //--++ locked variables are undeletable by the Student
							'locked',
							function(){
								frame.vars[this.blockSpec].isLocked = !frame.vars[this.blockSpec].isLocked;
							}, //--------
							frame.vars[this.blockSpec].isLocked,
							'click to make the variable udeletable to student',
							'click to make the variable normal to student',
							world.role=='STUDENT'
						);
					}
				}     
//==========================================================================================	
///////////////////// hide custom blocks ////////////////////////////////////////////////
				if (this.selector == 'evaluateCustomBlock' ) {  //----- if is a custom block
// +++++++++++ put only GLOBAL custom blocks in repository ++++++++++++++ 
					if (this.definition && (this.definition.isGlobal && !this.definition.isHidden)) {	 //--++ do not put hidden custom blocks in depository
						menu.addItem(
						   "put global custom block on depository",
						   'putOnDepository'
						);
					}
				}   
//////////////////////////////////////////////////////////////////////
				if (StageMorph.prototype.enableCodeMapping) {
					menu.addLine();
					menu.addItem(
						'header mapping...',
						'mapToHeader'
					);
					menu.addItem(
						'code mapping...',
						'mapToCode'
					);
				}
			}
		}
		if (this.selector == 'reportGetVar') {  //----- if is custom variable
////// +++++++++++ share only GLOBAL variables  ++++++++++++++ ////////////
			sprite = ide.currentSprite;
			frame = sprite.variables.parentFrame.silentFind(this.blockSpec);		
			//Network support
			if (NetworkClient.io !== null && NetworkClient.io.connected && frame){
				if (NetworkClient.sharedVariables[this.blockSpec]){ //if variable is shared, unshare
					menu.addItem(
						"unshare",
						function(){
							delete NetworkClient.sharedVariables[this.blockSpec];
							NetworkClient.io.emit("unshare_variable", this.blockSpec);
						}
					);
				} else{
						menu.addItem(
							"share...",
							function(){
								NetworkClient.sharedVariables[this.blockSpec] = true;
								NetworkClient.io.emit("share_variable",{ "data" :{"name":this.blockSpec,"value":frame.vars[this.blockSpec].value}});
							}
						);
					}
				}
						//Network support
		}

//v 4.0.10.2 - Rename variables
           if (this.selector === 'reportGetVar') {
               if (this.parent instanceof SyntaxElementMorph) {
                    menu.addLine();
                    menu.addItem(
                        'rename...',
                        function () {
                            myself.refactorThisVar(true); // just the template
                        },
                        'rename only\nthis reporter'
                    );
                    menu.addItem(
                        'rename all...',
                        'refactorThisVar',
                        'rename all blocks that\naccess this variable'
                    );
               } else{
                   rcvr = this.scriptTarget();
                    if (this.isInheritedVariable(false)) { // fully inherited
                        addOption(
                            'inherited',
                            function () {
                                rcvr.toggleInheritedVariable(myself.blockSpec);
                            },
                            true,
                            'uncheck to\ndisinherit',
                            null
                        );
                    } else { // not inherited
                        if (this.isInheritedVariable(true)) { // shadowed
                            addOption(
                                'inherited',
                                function () {
                                    rcvr.toggleInheritedVariable(myself.blockSpec);
                                },
                                false,
                                null,
                                localize('check to inherit\nfrom')
                                    + ' ' + rcvr.exemplar.name
                            );
                        }
                        addOption(
                            'transient',
                            'toggleTransientVariable',
                            myself.isTransientVariable(),
                            'uncheck to save contents\nin the project',
                            'check to prevent contents\nfrom being saved'
                        );
                        menu.addLine();
                        menu.addItem(
                            'rename...',
                            function () {
                                myself.refactorThisVar(true); // just the template
                            },
                            'rename only\nthis reporter'
                        );
                        menu.addItem(
                            'rename all...',
                            'refactorThisVar',
                            'rename all blocks that\naccess this variable'
                        );
                    }
               }               
           }
        //   if (!this.parent instanceof SyntaxElementMorph) {  // v 4.1
                if (StageMorph.prototype.enableInheritance) {
                    rcvr = this.scriptTarget();
                    field = {
                        xPosition: 'x position',
                        yPosition: 'y position',
                        direction: 'direction',
                        getScale: 'size',
                        getCostumeIdx: 'costume #'
                    }[this.selector];
                    if (field && rcvr && rcvr.exemplar) {
                        menu.addLine();
                        addOption(
                            'inherited',
                            function () {
                                rcvr.toggleInheritanceForAttribute(field);
                            },
                            rcvr.inheritsAttribute(field),
                            'uncheck to\ndisinherit',
                            localize('check to inherit\nfrom')
                                + ' ' + rcvr.exemplar.name
                        );
                    }
             }
       // }
                
// --- v 4.0.10.2
                
		return menu;
	}

/////////////////// copied from gui.js /////////////////////////////////
	function addPreference(label, toggle, test, onHint, offHint, hide) {
        var on = '\u2611 ',
            off = '\u2610 ';
        if (!hide) {
            menu.addItem(
                (test ? on : off) + localize(label),
                toggle,
                test ? onHint : offHint,
                hide ? new Color(100, 0, 0) : null
            );
        }
    }
///////////////// add draggable selection for dev mode /////////////////////
        //draggable is not available inside Hints editor, except top blocks 
        //if (!(this.parentThatIsA(HintsEditorMorph) && ((this.topBlock()!==this || this.selector=='demoCustomBlock') || this instanceof ReporterBlockMorph ))){
            addPreference(
                    'draggable',
                    'toggleIsDraggableBlock', //--------
                    this.isDraggable,
                    'click to make the block draggable to student',
                    'click to make the block undraggalbe to student',
                    world.role=='STUDENT' || (this.parentThatIsA(PrototypeHatBlockMorph) && !this.parentThatIsA(PrototypeHatBlockMorph).definition.isGlobal) //allow undraggable blocks, only in dev mode global scripts and global custom blocks
            );
        //}    
////////////////////////////////////////////////////////////////////////
  	
	if (world.role !== "STUDENT" && this instanceof HatBlockMorph) {  //hide-show block menu FOR SCRIPT BLOCKS, only for hat blocks-----------------
		if (this.isHidden) {
			menu.addItem(
				"show",
				function(){
					this.show();
				}
			);
		} else {
			menu.addItem(
				"hide",
				function(){
					this.hide();
				}
			);
		}
    } 
//----------------------------------------------------------------
	if (world.role !== 'STUDENT' || this.isDraggable) { // do not show menu if in Student role and is in draggable block
		if (!world.HOCmode){
			menu.addLine();
		}	
		if (ide.currentCategory!=='depository' || world.role !== 'STUDENT') {  //When student is in depository he can not rename vars or relabel commands
			if (this.selector === 'reportGetVar') {
				 menu.addItem(
                                    'rename...',
                                    renameVar,
                                    'rename only\nthis reporter'
                                );
			} else if (SpriteMorph.prototype.blockAlternatives[this.selector]) {
				menu.addItem(
					'relabel...',
					function () {
						myself.relabel(
							SpriteMorph.prototype.blockAlternatives[myself.selector]
						);
					}
				);
			} else if (this.definition && this.alternatives && world.role !== 'STUDENT') { // custom block, prevent renaming from STUDENT in case of hidden blocks
				alternatives = this.alternatives();
				if (alternatives.length > 0) {
					menu.addItem(
						'relabel...',
						function () {myself.relabel(alternatives); }
					);
				}
			}
		}
                if (world.role !== 'STUDENT' || !hasNestedUndraggables){ // if not in STUDENT mode with nested undraggables
                    menu.addItem(
                            "duplicate",
                            function () {
                                    var dup = myself.fullCopy(true), //true if for duplicateblock action
                                        ide = myself.parentThatIsA(IDE_Morph),
                                        blockEditor = myself.parentThatIsA(BlockEditorMorph);
                                    if (dup){ //if succeded(fails for cslot with undraggables)    
                                        dup.pickUp(world);
                                        // register the drop-origin, so the block can
                                        // slide back to its former situation if dropped
                                        // somewhere where it gets rejected
                                        if (!ide && blockEditor) {
                                            ide = blockEditor.target.parentThatIsA(IDE_Morph);
                                        }
                                        if (ide) {
                                                world.hand.grabOrigin = {
                                                        origin: ide.palette,
                                                        position: ide.palette.center()
                                                };
                                        }
                                    }
                            },
                            'make a copy\nand pick it up'
                    );
                    if (this instanceof CommandBlockMorph && this.nextBlock()) {
                            menu.addItem(
                                    (proc ? this.fullCopy() : this).thumbnail(0.5, 60),
                                    function () {
                                        var cpy = myself.fullCopy(true),//for nested undraggables
                                            nb,
                                            ide = myself.parentThatIsA(IDE_Morph),
                                            blockEditor = myself.parentThatIsA(BlockEditorMorph);
                                        if (cpy){    
                                            nb = cpy.nextBlock();
                                            if (nb) {nb.destroy(); }
                                            cpy.pickUp(world);
                                            if (!ide && blockEditor) {
                                                ide = blockEditor.target.parentThatIsA(IDE_Morph);
                                            }
                                            if (ide) {
                                                world.hand.grabOrigin = {
                                                    origin: ide.palette,
                                                    position: ide.palette.center()
                                                };
                                            }
                                        }    
                                    },
                                    'only duplicate this block'
                            );
                    }
                }
                //if (!hasNestedUndraggables){ //do not delete blocks with nested undraggables
                    menu.addItem(
                            "delete",
                            'userDestroy' //userdestroy with undraggables
                    );
                //}    
		if (!world.HOCmode){ // Do not show the following in HOC mode
			menu.addItem(
				"script pic...",
				function () {
					var ide = myself.parentThatIsA(IDE_Morph) ||
						myself.parentThatIsA(BlockEditorMorph).target.parentThatIsA(
							IDE_Morph
					);
					ide.saveCanvasAs(
						myself.topBlock().scriptPic(),
						ide.projetName || localize('Untitled') + ' ' +
							localize('script pic')
					);
				},
				'open a new window\nwith a picture of this script'
			);
                        if (shiftClicked) {
                            menu.addItem(
                                'download script',
                                function () {
                                    var ide = myself.parentThatIsA(IDE_Morph),
                                        blockEditor = myself.parentThatIsA(BlockEditorMorph);
                                    if (!ide && blockEditor) {
                                        ide = blockEditor.target.parentThatIsA(IDE_Morph);
                                    }
                                    if (ide) {
                                        ide.saveXMLAs(
                                            ide.serializer.serialize(myself),
                                            myself.selector + ' script',
                                            false);
                                    }
                                },
                                'download this script\nas an XML file',
                                new Color(100, 0, 0)
                                );
                        }
			if (proc) {
				if (vNames.length) {
					menu.addLine();
					vNames.forEach(function (vn) {
						menu.addItem(
							vn + '...',
							function () {
								proc.doShowVar(vn);
							}
						);
					});
				}
                                proc.homeContext.variables.names().forEach(function (vn) {
                                    if (!contains(vNames, vn)) {
                                        menu.addItem(
                                            vn + '...',
                                            function () {
                                                proc.doShowVar(vn);
                                            }
                                        );
                                    }
                                });
				return menu;
			}
			if (this.parent.parentThatIsA(RingMorph)) {
				menu.addLine();
				menu.addItem("unringify", 'unringify');                             
                                top = this.topBlock();
                                if (this instanceof ReporterBlockMorph ||
                                        (!(top instanceof HatBlockMorph))) {
                                    menu.addItem("ringify", 'ringify');
                                }    
				return menu;
			}
			if (this.parent instanceof ReporterSlotMorph
					|| (this.parent instanceof CommandSlotMorph)
					|| (this instanceof HatBlockMorph)
					|| (this instanceof CommandBlockMorph
						&& (this.topBlock() instanceof HatBlockMorph))) {
				return menu;
			}
			menu.addLine();
			menu.addItem("ringify", 'ringify');
			if (StageMorph.prototype.enableCodeMapping) {
				menu.addLine();
				menu.addItem(
					'header mapping...',
					'mapToHeader'
				);
				menu.addItem(
					'code mapping...',
					'mapToCode'
				);
			}
		}	
	}
    return menu;
};


// Toggle isdraggable for blocks ///////////////
BlockMorph.prototype.toggleIsDraggableBlock = function () {
	var status = !this.isDraggable;
	this.setDraggable(status);
}
///////////////////////////////////////

BlockMorph.prototype.hidePrimitive = function () {
    var ide = this.parentThatIsA(IDE_Morph),
        cat;
    if (!ide ) {return; }

    StageMorph.prototype.hiddenPrimitives[this.selector] = true;
     cat = {
        doWarp: 'control',
        reifyScript: 'operators',
        reifyReporter: 'operators',
        reifyPredicate: 'operators',
        doDeclareVariables: 'variables'
    }[this.selector] || this.category;
    if (cat === 'lists') {
		cat = 'variables'; 
	}
	this.deleteFromDepository();  //--++ delete from depository if exists
    ide.flushBlocksCache(cat);
    ide.refreshPalette();

};
//------- show primitive
BlockMorph.prototype.showPrimitive = function () {
    var ide = this.parentThatIsA(IDE_Morph),
		cat;
    if (!ide) {return; }
    delete StageMorph.prototype.hiddenPrimitives[this.selector];
        cat = {
        doWarp: 'control',
        reifyScript: 'operators',
        reifyReporter: 'operators',
        reifyPredicate: 'operators',
        doDeclareVariables: 'variables'
    }[this.selector] || this.category;
    if (cat === 'lists') {cat = 'variables'; }
	ide.flushBlocksCache(cat);
    ide.refreshPalette();
};
//----------------------

////// +++++++++ put on repository +++++++++++++ /////////////////
BlockMorph.prototype.putOnDepository = function () {
    var ide = this.parentThatIsA(IDE_Morph),
        stage, 
		block,idx;


	if (!ide ) {return; }
	stage = ide.stage;
	if (this.selector!='reportGetVar') {
		if (this.selector != 'evaluateCustomBlock'){
				block = ['primitive',this.selector,this.category];
		} else {
				block = ['custom-block', this.definition.blockSpec() ,this.definition.category];
			}
	} else {
			  block = ['variable', this.blockSpec, 'vars'];
	}
	idx = indexOf(stage.depositoryCommands, block);
	if (idx == -1) {		// if not already in depository, add the block
		stage.depositoryCommands.push(block);
	}
		ide.flushBlocksCache('depository');
	//	ide.flushPaletteCache();
	//	ide.refreshPalette();
};
//++++++++++++++++++++++++++++++++++++++++++++++
BlockMorph.prototype.deleteFromDepository = function () {
    var ide = this.parentThatIsA(IDE_Morph),
        stage,
		block,
		idx;

 	if (!ide ) {return; }
	stage = ide.stage;
	if (this.selector!='reportGetVar') {
		if (this.selector != 'evaluateCustomBlock'){
				block = ['primitive',this.selector,this.category];
	}
		else {
				block = ['custom-block', this.definition.blockSpec(),this.definition.category];
			}
	} else {
		block = ['variable', this.blockSpec, 'vars'];
	}
	idx = indexOf(stage.depositoryCommands, block);

	if (idx !== -1) {
        stage.depositoryCommands.splice(idx, 1);
        }

    ide.flushBlocksCache('depository');
	ide.flushPaletteCache('depository');
	ide.refreshPalette();
};
//////////////////////////////////////////////////////////////////

BlockMorph.prototype.addHighlight = function (oldHighlight) {
    var isHidden = !this.isVisible,
        highlight;

    if (isHidden) {this.show(); } 
    highlight = this.highlight(
        oldHighlight ? oldHighlight.color : this.activeHighlight,
        this.activeBlur,
        this.activeBorder
    );
    this.addBack(highlight);
    this.fullChanged();
    if (isHidden) {this.hide(world.role=='STUDENT'); } //--------- do not show hidden blocks when executed in student mode
    return highlight;
};

BlockMorph.prototype.addErrorHighlight = function () {
    var isHidden = !this.isVisible,
        highlight;

    if (isHidden) {this.show(); } 
    this.removeHighlight();
    highlight = this.highlight(
        this.errorHighlight,
        this.activeBlur,
        this.activeBorder
    );
    this.addBack(highlight);
    this.fullChanged();
    if (isHidden) {this.hide(world.role=='STUDENT'); } //--------- do not show hidden blocks when executed in student mode
    return highlight;
};

BlockMorph.prototype.fullCopy = function (duplicateBlock) {
    var ans = BlockMorph.uber.fullCopy.call(this),
		nextUndraggable;		
    ans.removeHighlight();
    ans.isDraggable = true;   
    if (this.instantiationSpec) {
        ans.setSpec(this.instantiationSpec);
    }
 
    function makeDraggable(block){ //helper, for making draggables deeply
        var  hasCSlot = block.children.find(function (child){
                                                return child instanceof CSlotMorph;
                                            });
        block.isDraggable = true;
        if (hasCSlot && hasCSlot.children.length>0){ //if i am a cslot command, check deeply
            hasCSlot.children[0].blockSequence().map(function(block){
                makeDraggable(block);
            });
        }
    }
    
///////////////////// handle the undraggable blocks
	if (duplicateBlock && this instanceof CommandBlockMorph) { // if duplicate block was called
		if (world.role!=='STUDENT'){ 	// TEACHERS mode, make all copied blocks draggable
			ans.blockSequence().map(function (block){
				 makeDraggable(block);
			});
		} else { // STUDENT mode, copy until the first undraggable block
                                              
			nextUndraggable = detect( //find the block in the stack, who's next block is undraggable, or the block who has nested undraggables
						ans.blockSequence(),
						function (child) {
							return child instanceof CommandBlockMorph
								&& (child.nextBlock() && (!child.nextBlock().isDraggable || child.nextBlock().hasNestedUndraggables()));
						}
					);		
			if (!isNil(nextUndraggable)){ //if exists                                                              
                            nextUndraggable.children.pop(); //the last element of the children array is the next block. Popping means, set it to null.
			}	
		}
	}	
///////////////////////////////////////////////////

//v 4.0.10.2
    ans.allChildren().filter(function (block) {
        if (block instanceof SyntaxElementMorph) {
            block.cachedInputs = null;
            if (block.isCustomBlock) {
                block.initializeVariables();
            }
        }
        return !isNil(block.comment);
    }).forEach(function (block) {
        var cmnt = block.comment.fullCopy();
        block.comment = cmnt;
        cmnt.block = block;
    });
    ans.cachedInputs = null;
  
    return ans;
//--- v 4.0.10.2
};

BlockMorph.prototype.mouseClickLeft = function () {
    var top = this.topBlock(),
        receiver = top.scriptTarget(),//receiver(),
        shiftClicked = this.world().currentKey === 16,
        stage;
    if (shiftClicked && !this.isTemplate) {
        return this.focus();
    }
    if (top instanceof PrototypeHatBlockMorph) {
        return top.mouseClickLeft();
    }    
    if (receiver) {
        stage = receiver.parentThatIsA(StageMorph);
        if (stage) {
			if (world.role == 'STUDENT' && stage.exerciseMode)  {return;}       //+++ Disable mouse click execution of blocks in exerciseMode in STUDENT
            stage.threads.toggleProcess(top,receiver);
        }
    }
};

//////////////////////////////////////////////////////////////////
// new help handling for custom blocks. priority is the picture, and then the comment
/////////////////////////////////////////////////////////////////
BlockMorph.prototype.showHelp = function () {
    var myself = this,
        ide = this.parentThatIsA(IDE_Morph),
        blockEditor, hintsEditor,
        pic = new Image(),
        help,
        comment,
        block,
        isCustomBlock = this.selector === 'evaluateCustomBlock',
        spec = (isCustomBlock && this.definition) ?
                this.definition.helpSpec() : this.selector,
        ctx;    
    if (!ide) {
        blockEditor = this.parentThatIsA(BlockEditorMorph);
        if (blockEditor) {
            ide = blockEditor.target.parentThatIsA(IDE_Morph);
        }
        hintsEditor = this.parentThatIsA(HintsEditorMorph);
        if (hintsEditor) {
            ide = hintsEditor.target.parentThatIsA(IDE_Morph);
        }
    }    
    pic.onload = function () {
        help = newCanvas(new Point(pic.width, pic.height));
        ctx = help.getContext('2d');
        ctx.drawImage(pic, 0, 0);
        new DialogBoxMorph().inform(
            'Help',
            null,
            myself.world(),
            help
        );
    };

    pic.onerror = function(){
        if (isCustomBlock && myself.definition.comment) {
            block = myself.fullCopy();
            block.addShadow();
            comment = myself.definition.comment.fullCopy();
            comment.contents.parse();
            help = '';
            comment.contents.lines.forEach(function (line) {
                help = help + '\n' + line;
            });
            new DialogBoxMorph().inform(
                'Help',
                help.substr(1),
                myself.world(),
                block.fullImage()
            );
        }
    }


    
   // } else {
        pic.src = ide.resourceURL('help', spec + '.png');
   // }
    
};

// ------------------------ snapHE.gr 4.1
// checks deeply, if a CSslot command block, has a nested undraggable command
CommandBlockMorph.prototype.hasNestedUndraggables = function(){
    var myself=this,
        hasCSlot = this.children.find(function (child){
            return child instanceof CSlotMorph;
        });
        
    if (!hasCSlot){ return false;}
    return hasCSlot.children.length>0 && detect(hasCSlot.children[0].blockSequence(),
                                                    function (child) {
                                                        return child instanceof CommandBlockMorph && (!child.isDraggable || child.hasNestedUndraggables());
                                                    }
                                         );           
}
// ------------------------

CommandBlockMorph.prototype.allAttachTargets = function (newParent) {
    var myself = this,
        target = newParent || this.parent,
        answer = [],
        topBlocks;

//v 4.0.10.2    
    if (this instanceof HatBlockMorph && newParent.rejectsHats) {
        return answer;
    }
// --- v 4.0.10.2
    topBlocks = target.children.filter(function (child) {
        return (child !== myself) &&
            child instanceof SyntaxElementMorph &&
            !child.isTemplate && child.isVisible; // NEW do not attach yourself on invisible blocks
    });
    topBlocks.forEach(function (block) {
        block.forAllChildren(function (child) {
            if (child.attachTargets) {
                child.attachTargets().forEach(function (at) {
                    answer.push(at);
                });
            }
        });
    });
    return answer;
};

//v 4.0.10.2 
CommandBlockMorph.prototype.snap = function (hand) {    
    var target = this.closestAttachTarget(),
        scripts = this.parentThatIsA(ScriptsMorph),
        before,
        next,
        offsetY,
        cslot,
        affected;

  
    scripts.clearDropInfo();   
    scripts.lastDroppedBlock = this;
    if (target === null) {
            if (! (this instanceof HatBlockMorph)) {this.show();} //if this is not a hat block, always show
        this.startLayout();
        this.fixBlockColor();
        this.endLayout();
        CommandBlockMorph.uber.snap.call(this); // align stuck comments
        if (hand) {            
            scripts.recordDrop(hand.grabOrigin);           
        }
        return;
    }

    scripts.lastDropTarget = target;

    this.startLayout();
    if (target.loc === 'bottom') {
        if (target.type === 'slot') {
            this.removeHighlight();
            scripts.lastNextBlock = target.element.nestedBlock();
            target.element.nestedBlock(this);
        } else {
            scripts.lastNextBlock = target.element.nextBlock();
            target.element.nextBlock(this);
        }
        if (this.isStop()) {
            next = this.nextBlock();
            if (next) {
                scripts.add(next);
                next.moveBy(this.extent().floorDivideBy(2));
                affected = this.parentThatIsA(CommandSlotMorph);
                if (affected) {
                    affected.fixLayout();
                }
            }
        }
    } else if (target.loc === 'top') {
        target.element.removeHighlight();
        offsetY = this.bottomBlock().bottom() - this.bottom();
        this.setBottom(target.element.top() + this.corner - offsetY);
        this.setLeft(target.element.left());
        this.bottomBlock().nextBlock(target.element);
    } else if (target.loc === 'wrap') {
        cslot = detect( // this should be a method making use of caching
            this.inputs(), // these are already cached, so maybe it's okay
            function (each) {return each instanceof CSlotMorph; }
        );
        // assume the cslot is (still) empty, was checked determining the target
        before = (target.element.parent);
        scripts.lastWrapParent = before;

        // adjust position of wrapping block
        this.moveBy(target.point.subtract(cslot.slotAttachPoint()));

        // wrap c-slot around target
        cslot.nestedBlock(target.element);
        if (before instanceof CommandBlockMorph) {
            before.nextBlock(this);
        } else if (before instanceof CommandSlotMorph) {
            before.nestedBlock(this);
        }

        // fix zebra coloring.
        // this could probably be generalized into the fixBlockColor mechanism
        target.element.blockSequence().forEach(
            function (cmd) {cmd.fixBlockColor(); }
        );
    }
    this.fixBlockColor();
    this.endLayout();
    CommandBlockMorph.uber.snap.call(this); // align stuck comments
// ------------------------------- if Im snaped in a hidden stack hide me, else show me
	if (this.parent.isHidden ) { 
		this.hide();
	} else if (this.nextBlock()) {
		if (this.isHidden && this instanceof HatBlockMorph ) { 
			this.hide();
		} else {
			this.show();
		}
	} else {
		this.show();
	}
// -------------------------------
    
    if (hand) {
        scripts.recordDrop(hand.grabOrigin);
    }
    if (this.snapSound) {
        this.snapSound.play();
    }
};

// --- v 4.0.10.2 
//---------------------------------------------------------
// ========== This where the handling of blocks been dropped on locked scripts is made ======================================

CommandBlockMorph.prototype.prepareToBeGrabbed = function (handMorph,forRedrop) {
//forRedrop is used when is called from recoverLastDrop in a redrop operation. Nothing is stored then.    
    var scripts = this.parentThatIsA(ScriptsMorph),
        cs = this.parentThatIsA(CommandSlotMorph),
        pb,
        nb,
        above,
        cslot = this.parentThatIsA(CSlotMorph), //possible cslot parent(i am nested i a cslot command)
        cslotChild, //i am a cslot command
	initDiff = handMorph.position().subtract(this.position()); //store the initial difference between block and handmorph positions because it might be changed and cause distortion

        nop(handMorph);       
        cslotChild = this.children.find(function(child){ //find my first cslot, if any(then i am a cslot command)
           return child instanceof CSlotMorph; 
        });

	if (this.parent) {
            pb = this.parent.parentThatIsA(CommandBlockMorph);
	}
	if (pb && (pb.nextBlock() === this)) {
            above = pb;
	} else if (cs && (cs.nestedBlock() === this)) {
            above = cs;
	}
        if (this.hasNestedUndraggables()){ //if I am CSlot command with nested undraggables            
             cslotChild = this.children.find(function(child){ //find my first cslot
                            return child instanceof CSlotMorph; 
                          });           
                nb = cslotChild.children[0]; //get the first of my nested blocks                          
                if (nb){                                     
                    if(scripts && !scripts.dropRecord.undropInfo && !forRedrop){// && isNil(scripts.dropRecord.lastRecord)){                        //
 //store undrop data for nested undraggables, in order to undrop properly
                        scripts.dropRecord.undropInfo = {
                            lastDroppedBlock : this,
                            lastWrapParent : this.parent,
                            lastDropTarget : nb,
                            lastNextBlock : this.nextBlock()
                        }
                         
                    }
                 
                    if (this.nextBlock()){ //if I have a next block
                        nb.bottomBlock().nextBlock(this.nextBlock());
                    }    
                    if (above instanceof CommandSlotMorph) { //if I am nested inside another cslot
                        above.nestedBlock(nb);
                    } else if (above instanceof CommandBlockMorph) {
                        above.nextBlock(nb);
                    } else {
                            if (scripts){
                                scripts.add(nb);
                            }     
                    }
                    
                    nb.blockSequence().forEach( //fix zebra coloring
                        function (cmd) {cmd.fixBlockColor(); }
                    );
                    cslotChild.fixLayout(); //fix my layout
                    if (cslot) {
                        cslot.fixLayout(); //fix my parent's cslot layout
                    }
                    
                    this.removeShadow();
                    this.setPosition(handMorph.position().subtract(initDiff)); //restore the diffence in position between block and handmorph                  
                    
                    return; // terminate here
                }    
           
        } 
        //if I am a normal command (including cslot with no nested undraggables
            if (this.nextBlock()){
                nb = detect(
                            this.blockSequence(),
                            function (child) {
                                return child instanceof CommandBlockMorph
                                       && (!child.isDraggable || child.hasNestedUndraggables());
                            }
                    );
                if (nb){
                    if (above instanceof CommandSlotMorph) {
                        above.nestedBlock(nb);
                    } else if (above instanceof CommandBlockMorph) {
                        above.nextBlock(nb);
                        } else {
                            if (scripts){
                                scripts.add(nb);
                            }
                        }
                } else if (cslot) {
                    cslot.fixLayout();
                }

                this.removeShadow();
                this.setPosition(handMorph.position().subtract(initDiff)); //restore the diffence in position between block and handmorph
            }
   
	
	CommandBlockMorph.uber.prepareToBeGrabbed.call(this, handMorph);
};

// -------------------------------------------------------- 
//overriden to deal with undrop, for undraggable nested blocks
ScriptsMorph.prototype.undrop = function () {
    var myself = this;    
    if (this.isAnimating) {return; }
    if (!this.dropRecord || !this.dropRecord.lastRecord) {return; }
    if (!this.dropRecord.situation) {
        this.dropRecord.situation =
            this.dropRecord.lastDroppedBlock.situation();
    }
    this.isAnimating = true;
    this.dropRecord.lastDroppedBlock.slideBackTo(
        this.dropRecord.lastOrigin,
        null,
        this.recoverLastDrop(),
        function () {
            myself.updateToolbar();
            myself.isAnimating = false;
        }
    );
    this.dropRecord = this.dropRecord.lastRecord;
};

ScriptsMorph.prototype.recoverLastDrop = function (forRedrop) {
    // retrieve the block last touched by the user and answer a function
    // to be called after the animation that moves it back right before
    // dropping it into its former situation
    var rec = this.dropRecord,
        dropped,
        onBeforeDrop,
        parent;

 
 
    if (!rec || !rec.lastDroppedBlock) {
        throw new Error('nothing to undrop');
    }
    dropped = rec.lastDroppedBlock;
    parent = dropped.parent;   
    if (dropped instanceof CommandBlockMorph) {
        if (rec.lastNextBlock) {
            if (rec.action === 'delete') {
                if (forRedrop) {
                    this.add(rec.lastNextBlock);
                }
            } else {               
                this.add(rec.lastNextBlock);
            }
        }
        if (rec.lastDropTarget) {
            if (rec.lastDropTarget.loc === 'bottom') {
                if (rec.lastDropTarget.type === 'slot') {
                    if (rec.lastNextBlock) {
                        rec.lastDropTarget.element.nestedBlock(
                            rec.lastNextBlock
                        );
                    }
                } else { // 'block'
                    if (rec.lastNextBlock) {
                        rec.lastDropTarget.element.nextBlock(
                            rec.lastNextBlock
                        );
                    }
                }
            } else if (rec.lastDropTarget.loc === 'top') {
                this.add(rec.lastDropTarget.element);
            } else if (rec.lastDropTarget.loc === 'wrap') {
                var cslot = detect( // could be cached...
                    rec.lastDroppedBlock.inputs(), // ...although these are
                    function (each) {return each instanceof CSlotMorph; }
                );
                if (rec.lastWrapParent instanceof CommandBlockMorph) {
                    if (forRedrop) {                      
                        onBeforeDrop = function () {
                            cslot.nestedBlock(rec.lastDropTarget.element);                          
                        };
                    } else {
                        rec.lastWrapParent.nextBlock(
                            rec.lastDropTarget.element
                        );
                    }
                } else if (rec.lastWrapParent instanceof CommandSlotMorph) {
                    if (forRedrop) {
                        onBeforeDrop = function () {
                            cslot.nestedBlock(rec.lastDropTarget.element);
                        };
                    } else {
                        rec.lastWrapParent.nestedBlock(
                            rec.lastDropTarget.element
                        );
                    }
                } else {
                    this.add(rec.lastDropTarget.element);
                }

                // fix zebra coloring.
                // this could be generalized into the fixBlockColor mechanism
                rec.lastDropTarget.element.blockSequence().forEach(
                    function (cmd) {cmd.fixBlockColor(); }
                );
                cslot.fixLayout();
            }
        }
// ----------------------------------------              
          if (!forRedrop && rec.lastRecord && rec.lastRecord.undropInfo){
//if we are in an udrop situation, of a slot block with nested undraggables              
            var cslot = detect(
                    rec.lastRecord.undropInfo.lastDroppedBlock.inputs(), 
                    function (each) {return each instanceof CSlotMorph; }
                ),
                element = rec.lastRecord.undropInfo.lastDropTarget || null,
                nextBlock = rec.lastRecord.undropInfo.lastNextBlock || null,
                blockParent = rec.lastRecord.undropInfo.lastWrapParent || null,
                block = rec.lastRecord.undropInfo.lastDroppedBlock;
                                
            onBeforeDrop = function(){
//if it has undrop info, restore previous state              
                if (element){
                    cslot.nestedBlock(element);
                    block.nextBlock(nextBlock);
                    cslot.fixLayout();
                }
            }
        }
        
// ---------------------------------------        
    } else if (dropped instanceof ReporterBlockMorph) {         
        if (rec.lastDropTarget) {         
            rec.lastDropTarget.replaceInput(
                rec.lastDroppedBlock,
                rec.lastReplacedInput
                //rec.lastReplacedInput,
                //rec.lastDroppedBlock
            );   
            rec.lastDropTarget.fixBlockColor(null, true);
            if (rec.lastPreservedBlocks) {
                rec.lastPreservedBlocks.forEach(function (morph) {
                    morph.destroy();
                });
            }
        }
    } else if (dropped instanceof CommentMorph) {
        if (forRedrop && rec.lastDropTarget) {
            onBeforeDrop = function () {
                rec.lastDropTarget.element.comment = dropped;
                dropped.block = rec.lastDropTarget.element;
                dropped.align();
            };
        }
    } else {
        throw new Error('unsupported action for ' + dropped);
    }
    this.clearDropInfo();
    dropped.prepareToBeGrabbed(this.world().hand,forRedrop); // -- do not store undropinfo forRedrop 
    if (dropped instanceof CommentMorph) {
        dropped.removeShadow();
    }
    this.add(dropped);
    parent.reactToGrabOf(dropped);
    if (dropped instanceof ReporterBlockMorph && parent instanceof BlockMorph) {
        parent.changed();
    }
    if (rec.action === 'delete') {
        if (forRedrop && rec.lastNextBlock) {
            if (parent instanceof CommandBlockMorph) {
                parent.nextBlock(rec.lastNextBlock);
            } else if (parent instanceof CommandSlotMorph) {
                parent.nestedBlock(rec.lastNextBlock);
            }
        }

        // animate "undelete"
        if (!forRedrop) {
            dropped.moveBy(new Point(-100, -20));
        }
    }


    return onBeforeDrop;
};
// --- overriden to deal with nested undraggables
originalUserDestroy = CommandBlockMorph.prototype.userDestroy;
CommandBlockMorph.prototype.userDestroy = function(){
    if (this.hasNestedUndraggables()){
        this.prepareToBeGrabbed(this.world().hand);
    }
    originalUserDestroy.call(this);
}
//  --------------------------------------
//v 4.0.10.2 
ReporterBlockMorph.prototype.snap = function (hand) {
    // passing the hand is optional (for when blocks are dragged & dropped)
    var scripts = this.parent,
        nb,
        target;

    this.cachedSlotSpec = null;
    if (!(scripts instanceof ScriptsMorph)) {
        return null;
    }

    scripts.clearDropInfo();
    scripts.lastDroppedBlock = this;

    target = scripts.closestInput(this, hand);    
    if (target !== null) {
        scripts.lastReplacedInput = target;
        scripts.lastDropTarget = target.parent;
        if (target instanceof MultiArgMorph) {
            scripts.lastPreservedBlocks = target.inputs();
            scripts.lastReplacedInput = target.fullCopy();
        } else if (target instanceof CommandSlotMorph) {
            scripts.lastReplacedInput = target;
            nb = target.nestedBlock();
            if (nb) {
                nb = nb.fullCopy();
                scripts.add(nb);
                nb.moveBy(nb.extent());
                nb.fixBlockColor();
                scripts.lastPreservedBlocks = [nb];
            }
        }
        target.parent.replaceInput(target, this);
        if (this.snapSound) {
            this.snapSound.play();
        }
    }
    this.startLayout();
    this.fixBlockColor();
    this.endLayout();
    ReporterBlockMorph.uber.snap.call(this);
// -------------------------------
	if (this.parent.isHidden ) { this.hide();}
	else {this.show();}
// -------------------------------
    if (hand) {
        scripts.recordDrop(hand.grabOrigin);
    }
};
// --- v 4.0.10.2 
ScriptsMorph.prototype.userMenu = function () {

//v 4.0.10.2 
    var menu = new MenuMorph(this),
        ide = this.parentThatIsA(IDE_Morph),
        shiftClicked = this.world().currentKey === 16,
        blockEditor,
        myself = this,
        obj = this.scriptTarget(),
        hasUndropQueue,
        stage = obj.parentThatIsA(StageMorph);

    function addOption(label, toggle, test, onHint, offHint) {
        var on = '\u2611 ',
            off = '\u2610 ';
        menu.addItem(
            (test ? on : off) + localize(label),
            toggle,
            test ? onHint : offHint
        );
    }    
    
    if (!ide) {
        blockEditor = this.parentThatIsA(BlockEditorMorph);
        if (blockEditor) {
            ide = blockEditor.target.parentThatIsA(IDE_Morph);
        }
    }
    if (this.dropRecord) {
        if (this.dropRecord.lastRecord) {
            hasUndropQueue = true;
            menu.addPair(
                [
                    new SymbolMorph(
                        'turnBack',
                        MorphicPreferences.menuFontSize
                    ),
                    localize('undrop')
                ],
                'undrop',
                '^Z',
                'undo the last\nblock drop\nin this pane'
            );
        }
        if (this.dropRecord.nextRecord) {
            hasUndropQueue = true;
            menu.addPair(
                [
                    new SymbolMorph(
                        'turnForward',
                        MorphicPreferences.menuFontSize
                    ),
                    localize('redrop')
                ],
                'redrop',
                '^Y',
                'redo the last undone\nblock drop\nin this pane'
            );
        }
        if (hasUndropQueue) {
            if (shiftClicked) {
                menu.addItem(
                    "clear undrop queue",
                    function () {
                        myself.dropRecord = null;
                        myself.clearDropInfo();
                        myself.recordDrop();
                    },
                    'forget recorded block drops\non this pane',
                    new Color(100, 0, 0)
                );
            }
            menu.addLine();
        }
    }
    if (!world.HOCmode){ // not visible in HOC mode
        menu.addItem('clean up', 'cleanUp', 'arrange scripts\nvertically');
        menu.addItem('add comment', 'addComment');    
        menu.addItem(
            'scripts pic...',
            'exportScriptsPicture',
            'open a new window\nwith a picture of all scripts'
        );
    }    
// --- v 4.0.10.2 
    if (ide) {
		if (world.role !== 'STUDENT' || (ide.currentCategory !== 'depository' && !ide.stage.noCustomBlocks)) {//do not allow student to create custom blocks when in depository	
			if (!world.HOCmode || this.lastDroppedBlock) {
				menu.addLine();
			}
                       if (!blockEditor && obj.exemplar) {
                                addOption(
                                    'inherited',
                                    function () {
                                        obj.toggleInheritanceForAttribute('scripts');
                                    },
                                    obj.inheritsAttribute('scripts'),
                                    'uncheck to\ndisinherit',
                                    localize('check to inherit\nfrom')
                                        + ' ' + obj.exemplar.name
                                );
                        }
			menu.addItem(
				'make a block...',
				function () {
					new BlockDialogMorph(
						null,
						function (definition) {
							 if (definition.spec !== '' && !(world.role=="STUDENT" && definition.spec.indexOf("$$")==0)) { //handle hiddens, do not allow student to create a hidden block(starting with $$)
								if (definition.isGlobal) {
									stage.globalBlocks.push(definition);
								} else {
									obj.customBlocks.push(definition);
								}
								if (definition.spec.indexOf("$$")==0) { //handle hiddens, hide the block if it starts with $$
									definition.isHidden = true;
									 ide.flushBlocksCache();
									} //handle hiddens end
								ide.flushPaletteCache();
								ide.refreshPalette();
								new BlockEditorMorph(definition, obj).popUp();
							} else if (definition.spec.indexOf("$$")==0) { myself.inform("Custom block name can not start with $$");} //handle hiddens
						},
						myself
					).prompt(
						'Make a block',
						null,
						myself.world()
					);
				}
			);
		}	
    }
    return menu;
};

InputSlotMorph.prototype.getVarNamesDict = function () {
    var block = this.parentThatIsA(BlockMorph),
        rcvr,
        tempVars = [],
        dict,
		ide,
		frame;

    if (!block) {
        return {};
    }
    rcvr = block.scriptTarget();
	block.allParents().forEach(function (morph) {
        if (morph instanceof PrototypeHatBlockMorph) {
            tempVars.push.apply(
                tempVars,
                morph.variableNames()
            );
            tempVars.push.apply(
                tempVars,
                morph.inputs()[0].inputFragmentNames()
            );
        } else if (morph instanceof BlockMorph) {
            morph.inputs().forEach(function (inp) {
                if (inp instanceof TemplateSlotMorph) {
                    tempVars.push(inp.contents());
                } else if (inp instanceof MultiArgMorph) {
                    inp.children.forEach(function (m) {
                        if (m instanceof TemplateSlotMorph) {
                            tempVars.push(m.contents());
                        }
                    });
                }
            });
        }
    });

    if (rcvr) {
//==================================== hide the hidden vars from the dictionary =========================		
	if (world.role !== 'STUDENT') {
		dict = rcvr.variables.allNamesDict();
	} else {
		dict = rcvr.variables.allVisibleNamesDict();
	}
//=======================================================================================================
        tempVars.forEach(function (name) {
			dict[name] = name; 
			});
        return dict;
    }
    return {};
};

//changed for making readonly custom block parameters, static
InputSlotMorph.prototype.setChoices = function (dict, readonly) {
    // externally specify choices and read-only status,
    // used for custom blocks
    var cnts = this.contents();
    this.choices = dict;
    this.isReadOnly = readonly || false;

// if a custom block parameter is readonly make it static, that is not to accept drops	
	if (this.parent instanceof CustomCommandBlockMorph || this.parent instanceof CustomReporterBlockMorph){
		this.isStatic = readonly;
	}
//////////////////////////////////////////

    if (this.parent instanceof BlockMorph) {
        this.parent.fixLabelColor();
        if (!readonly) {
            cnts.shadowOffset = new Point();
            cnts.shadowColor = null;
            cnts.setColor(new Color(0, 0, 0));
        }
    }
    this.fixLayout();
};




InputSlotMorph.prototype.setContents = function (aStringOrFloat) {
    var cnts = this.contents(),
        dta = aStringOrFloat,
        isConstant = dta instanceof Array;
    if (isConstant) {
        dta = localize(dta[0]);
        cnts.isItalic = !this.isReadOnly;
    } else { // assume dta is a localizable choice if it's a key in my choices
        cnts.isItalic = false;
        if (this.choices !== null && this.choices[dta] instanceof Array) {
            return this.setContents(this.choices[dta]);
        }
    }
    cnts.text = dta;
    if (isNil(dta)) {
        cnts.text = '';
    } else if (dta.toString) {
        cnts.text = dta.toString();
    }
	if (this.locked){ //@@ if locked(MessageReceive block) it appears in student as a label
		cnts.fontName = this.labelFontName;
        cnts.fontStyle = this.labelFontStyle;
        cnts.color = new Color(255, 255, 255);
        cnts.isBold = true;
		cnts.isShowingBlanks = false;
	}   //@@
    cnts.drawNew();

    // adjust to zebra coloring:
    if (this.isReadOnly && (this.parent instanceof BlockMorph)) {
        this.parent.fixLabelColor();
    }

    // remember the constant, if any
    this.constant = isConstant ? aStringOrFloat : null;
};


// v 4.0.10.2 
InputSlotMorph.prototype.menuFromDict = function (choices, noEmptyOption,enableKeyboard) {
    var key,
        menu = new MenuMorph(
            this.setContents,
            null,
            this,
            this.fontSize
        );

    if (choices instanceof Function) {
        choices = choices.call(this);
    } else if (isString(choices)) {
        choices = this[choices](enableKeyboard);
        if (!choices) { // menu has already happened
            return;
        }
    }
    if (!noEmptyOption) {
        menu.addItem(' ', null);
    }
    for (key in choices) {
        if (Object.prototype.hasOwnProperty.call(choices, key)) {
            if (key[0] === '~') {
                menu.addLine();
            // } else if (key.indexOf('§_def') === 0) {
            //     menu.addItem(choices[key].blockInstance(), choices[key]);
            } else if (choices[key] instanceof Object &&
                    !(choices[key] instanceof Array) &&
                    (typeof choices[key] !== 'function')) {
                menu.addMenu(key, this.menuFromDict(choices[key], true));
            } else {
                menu.addItem(key, choices[key],null,null,false,false,null,this.notLocal);  //++++++++++++++++++++++ localize contents or not
            }
        }
    }
    return menu;
};
// --- v 4.0.10.2 

InputSlotMorph.prototype.collidablesMenu = function () {
    var dict = {
            'mouse-pointer' : ['mouse-pointer'],
            edge : ['edge'],
            'pen trails' : ['pen trails']
        },
        rcvr = this.parentThatIsA(BlockMorph).scriptTarget(),//receiver(),
        stage = rcvr.parentThatIsA(StageMorph),
        allNames = [];

    stage.children.forEach(function (morph) {
        if (morph instanceof SpriteMorph && !morph.isClone) {
			if (world.role !== 'STUDENT' || !morph.isSecret) {       //do not show hidden morphs on the list for student
				if (morph.name !== rcvr.name) {
					allNames = allNames.concat(morph.name);
				}
			}	
        }
    });
    if (allNames.length > 0) {
        dict['~'] = null;
        allNames.forEach(function (name) {
            dict[name] = name;
        });
    }
    return dict;
};

InputSlotMorph.prototype.distancesMenu = function () {
    var dict = {
            'mouse-pointer' : ['mouse-pointer']
        },
        rcvr = this.parentThatIsA(BlockMorph).scriptTarget(),//receiver(),
        stage = rcvr.parentThatIsA(StageMorph),
        allNames = [];

    stage.children.forEach(function (morph) {
        if (morph instanceof SpriteMorph) {
			if (world.role !== 'STUDENT' || !morph.isSecret) {       //do not show hidden morphs on the list for student
				if (morph.name !== rcvr.name) {
					allNames = allNames.concat(morph.name);
				}
			}
        }
    });
    if (allNames.length > 0) {
        dict['~'] = null;
        allNames.forEach(function (name) {
            dict[name] = name;
        });
    }
    return dict;
};

InputSlotMorph.prototype.objectsMenu = function () {
    var rcvr = this.parentThatIsA(BlockMorph).scriptTarget(), //receiver(),
        stage = rcvr.parentThatIsA(StageMorph),
        dict = {},
        allNames = [];

    if (world.role !== 'STUDENT' || !stage.isSecret) {       //do not show hidden stage on the list for student
	dict[stage.name] = stage.name;
	}
    stage.children.forEach(function (morph) {
		if (world.role !== 'STUDENT' || !morph.isSecret) {       //do not show hidden morphs on the list for student
			if (morph instanceof SpriteMorph) {
				allNames.push(morph.name);
			}
		}
    });
    if (allNames.length > 0) {
        dict['~'] = null;
        allNames.forEach(function (name) {
            dict[name] = name;
        });
    }
    return dict;
};

InputSlotMorph.prototype.drawNew = function () {
    var context, borderColor, r;

    // initialize my surface property
    this.image = newCanvas(this.extent());
    context = this.image.getContext('2d');
    if (this.cachedNormalColor) { // if flashing
        borderColor = this.color;
    } else if (this.parent) {
        borderColor = this.parent.color;
    } else {
        borderColor = new Color(120, 120, 120);
    }
    context.fillStyle = this.color.toString();
    if (this.isReadOnly) {
        context.fillStyle = borderColor.darker().toString();       
		if (this.locked) {  	//--@@-- do not alter the borderColor when locked
			context.fillStyle = borderColor;
		}						//--@@--
    }

    // cache my border colors
    this.cachedClr = borderColor.toString();
    this.cachedClrBright = borderColor.lighter(this.contrast)
        .toString();
    this.cachedClrDark = borderColor.darker(this.contrast).toString();

    if (!this.isNumeric) {
        context.fillRect(
            this.edge,
            this.edge,
            this.width() - this.edge * 2,
            this.height() - this.edge * 2
        );
        if (!MorphicPreferences.isFlat && !this.locked) {  // --@@-- isLocked has flat design
            this.drawRectBorder(context);           
        }
    } else {
        r = (this.height() - (this.edge * 2)) / 2;
        context.beginPath();
        context.arc(
            r + this.edge,
            r + this.edge,
            r,
            radians(90),
            radians(-90),
            false
        );
        context.arc(
            this.width() - r - this.edge,
            r + this.edge,
            r,
            radians(-90),
            radians(90),
            false
        );
        context.closePath();
        context.fill();
        if (!MorphicPreferences.isFlat) {
            this.drawRoundBorder(context);
        }
    }
};

MultiArgMorph.prototype.init = function (
    slotSpec,
    labelTxt,
    min,
    eSpec,
    arrowColor,
    labelColor,
    shadowColor,
    shadowOffset,
    isTransparent
) {
    var label,
        arrows = new FrameMorph(),
        leftArrow,
        rightArrow,
        i;

    this.slotSpec = slotSpec || '%s';
    this.labelText =  localize(labelTxt || '');
    this.minInputs = min || 0;
    this.elementSpec = eSpec || null;
    this.labelColor = labelColor || null;
    this.shadowColor = shadowColor || null;
    this.shadowOffset = shadowOffset || null;

    this.canBeEmpty = true;
    MultiArgMorph.uber.init.call(this, null, true); // silently v 4.0.10.2

    // MultiArgMorphs are transparent by default b/c of zebra coloring
    this.alpha = isTransparent === false ? 1 : 0;
    arrows.alpha = isTransparent === false ? 1 : 0;
    arrows.noticesTransparentClick = true;
    this.noticesTransparentclick = true;

    // label text:
    label = this.labelPart(this.labelText);
    this.add(label);
	if (this.children.length>2) {  	//@@@@@@@@@@@ start nasty MultiArgsMorph bug
		label.hide(); 
	} else {
		label.isVisible = false;  
		label.changed();    
	}     							//@@@@@@@@@@@@ end

    // left arrow:
    leftArrow = new ArrowMorph(
        'left',
        this.fontSize,
        Math.max(Math.floor(this.fontSize / 6), 1),
        arrowColor
    );

    // right arrow:
    rightArrow = new ArrowMorph(
        'right',
        this.fontSize,
        Math.max(Math.floor(this.fontSize / 6), 1),
        arrowColor
    );

    // control panel:
    arrows.add(leftArrow);
    arrows.add(rightArrow);
    arrows.drawNew();
    arrows.acceptsDrops = false;

    this.add(arrows);

    // create the minimum number of inputs
    for (i = 0; i < this.minInputs; i += 1) {
        this.addInput();
    }
};

MultiArgMorph.prototype.hide =function () {
  //  this.isVisible = false;
  //  this.changed();
  MultiArgMorph.uber.hide.call(this);
};

MultiArgMorph.prototype.show =  function () {
  //  this.isVisible = true;
  //  this.changed();
  MultiArgMorph.uber.show.call(this);
};


MultiArgMorph.prototype.fixLayout = function () {
    if (this.slotSpec === '%t') {
        this.isStatic = true; // in this case I cannot be exchanged
    }
    if (this.parent) {
        var label = this.label(), shadowColor, shadowOffset;
        this.color = this.parent.color;
        shadowColor = this.shadowColor ||
            this.parent.color.darker(this.labelContrast);
        shadowOffset = this.shadowOffset || label.shadowOffset;
        this.arrows().color = this.color;
		if (!this.isHidden){ //@@@@@@@@@@ start nasty MultiArgsMorph bug
			if (this.labelText !== '') {
				if (!label.shadowColor.eq(shadowColor)) {
					label.shadowColor = shadowColor;
					label.shadowOffset = shadowOffset;
					label.drawNew();
				}
			}
		} //@@@@@@@@@@@ end
    }
    this.fixArrowsLayout();
    MultiArgMorph.uber.fixLayout.call(this);
    if (this.parent) {
        this.parent.fixLayout();
    }
};

MultiArgMorph.prototype.fixArrowsLayout = function () {
    var label = this.label(),
        arrows = this.arrows(),
        leftArrow = arrows.children[0],
        rightArrow = arrows.children[1],
        dim = new Point(rightArrow.width() / 2, rightArrow.height());
    if (this.inputs().length < (this.minInputs + 1)) {
		label.isVisible = false;  //@@@@@@@@@@@@@@ start nasty MultiArgsMorph bug
		label.changed();
		leftArrow.isVisible = false;
		leftArrow.changed();//}          //@@@@@@@@@@@@@@@@ end
       // leftArrow.hide();
        rightArrow.setPosition(
            arrows.position().subtract(new Point(dim.x, 0))
        );
        arrows.setExtent(dim);
    } else {
        if (this.labelText !== '') {
			if (this.children.length>2 && !this.isHidden) { //@@@@@@@@@@@@@@ start nasty MultiArgsMorph bug
				label.show();
			} else {
				if (this.isHidden) {
					label.alpha = 0.4;
					label.isHidden= true;
				}
				label.isVisible = true;
				label.changed();
			}   
		}
		if (this.children.length>2 && !this.isHidden) {
			leftArrow.show();
		} else {
			if (this.isHidden) {
				leftArrow.alpha = 0.4;
				leftArrow.isHidden= true;
			}
			leftArrow.isVisible = true;
			leftArrow.changed();
		}  														//@@@@@@@@@@@@@@ end
       // leftArrow.show();
        rightArrow.setPosition(leftArrow.topCenter());
        arrows.bounds.corner = rightArrow.bottomRight().copy();
    }
    arrows.drawNew();
};

MultiArgMorph.prototype.addInput = function (contents) {
    var i, name,
        newPart = this.labelPart(this.slotSpec),
        idx = this.children.length - 1;
    // newPart.alpha = this.alpha ? 1 : (1 - this.alpha) / 2;
    if (contents) {
        newPart.setContents(contents);
    } else if (this.elementSpec === '%scriptVars' ||
            this.elementSpec === '%blockVars') {
        name = '';
        i = idx;
        while (i > 0) {
            name = String.fromCharCode(97 + (i - 1) % 26) + name;
            i = Math.floor((i - 1) / 26);
        }
        newPart.setContents(name);
    } else if (contains(['%parms', '%ringparms'], this.elementSpec)) {
        newPart.setContents('#' + idx);
    }
    newPart.parent = this;
    this.children.splice(idx, 0, newPart);
    newPart.drawNew();
	if (this.isHidden) { //@@@@@@@@@@@@@@@@
		newPart.hide();
	}  
    this.fixLayout();
};

//============== Network support ============
// Network users menu
InputSlotMorph.prototype.usersMenu = function (inDict) { // Network users menu
    var dict = inDict || {}, i=0;
	for(id in NetworkClient.users){
		dict[ i+":"+ NetworkClient.users[id].nickname ]= [ i+":"+ NetworkClient.users[id].nickname, id];
		i++;
	}
    return dict;
};

InputSlotMorph.prototype.nVarsMenu = function () { //Network variables menu
	var block = this.parentThatIsA(BlockMorph),
        userName = block.inputs()[1].evaluate()[1],
		dict={};
	if (!userName){
		return dict;
	}
	
	for (v in NetworkClient.users[userName].variables){
		dict[v] = v;
	}		
	return dict;	
}

InputSlotMorph.prototype.sharedVarsMenu = function () { //shared variables menu
	var ide = this.parentThatIsA(IDE_Morph),
		globalVarsFrame,
		dict = {};
		if (ide){			
			globalVarsFrame = ide.stage.variables.parentFrame;
			if (globalVarsFrame){			
				for (v in globalVarsFrame.vars){					
					if (NetworkClient.sharedVariables[v]){//(globalVarsFrame.vars[v].isShared){
						dict[v] = [v, globalVarsFrame.vars ];
					}
				}
			}
		}
	return dict;	
}
// ========= Network support

