// ///////////// DEMO blocks and vars /////////////
//  These blocks create live animations of variable and custom block creations.
//  They DO NOT CREATE any actual variable or custom block.

// //////////// DUMMY blocks /////////////////////
// These blocks create any kind of block, using the specification in their input.
// The blocks created do nothing. They are DUMMY.
// They ARE SERIALIZED, retaining their spec.
// Usefull for demo puproses.
// eg. in a hint script, in place of custom blocks not yet created.
// For input types in block specification, see blocks.js.


//New label parts, for the new blocks
orSyntaxElementMorphLabelPart = SyntaxElementMorph.prototype.labelPart;
SyntaxElementMorph.prototype.labelPart = function (spec) {
    var part, myself=this;
    switch (spec) {
        case '%scope':
            part = new InputSlotMorph(
                    null,
                    false,
                    {
                        global: 'global',
                        local: 'local'                    
                    },
                    true
                );
            break;
        case '%blockType': 
            part = new InputSlotMorph(
                    null,
                    false,
                    {
                        command: 'command',
                        reporter: 'reporter',
                        predicate: 'predicate'
                    },
                    true
                );
            break;
        case '%blockCat':
            part = new InputSlotMorph(
                null,
                false,
                'blockCategories',
                true
            )
            break;  
        default:
            part = orSyntaxElementMorphLabelPart.call(this,spec);
    }	
    return part;
}

InputSlotMorph.prototype.blockCategories = function(){
    var dict = {},categories;
    //visible categrories, excluding depository
    categories = SpriteMorph.prototype.categories().filter(function (cat){
        return cat !== 'depository' && SpriteMorph.prototype.hiddenStandardCategories.indexOf(cat) < 0;
    });
    categories.forEach(function(name){
        dict[name] = name;
    });
    return dict;
}

//block creation
orSpriteMorphinitBlocks = SpriteMorph.prototype.initBlocks;
SpriteMorph.prototype.initBlocks = function () {   
    orSpriteMorphinitBlocks();
     // //////// DEMO blocks ///////////
    SpriteMorph.prototype.blocks.demoVariable = {
        type: 'hat',
        category: 'other',
        spec: 'demo variable %s of scope %scope'
    };
    SpriteMorph.prototype.blocks.demoCustomBlock = {
        type: 'hat',
        category: 'other',
        spec: 'demo custom block %s type: %blockType category: %blockCat scope %scope %parms'
    };
    // //////// dummy blocks ///////////
    SpriteMorph.prototype.blocks.dummyCommand = {
        type: 'command',
        category: 'other',
        spec: 'dummy command %s',
        defaults: ['type spec here']
    };
    SpriteMorph.prototype.blocks.dummyReporter = {
        type: 'reporter',
        category: 'other',
        spec: 'dummy reporter %s',
        defaults: ['type spec here']
    };
    SpriteMorph.prototype.blocks.dummyPredicate = {
        type: 'predicate',
        category: 'other',
        spec: 'dummy predicate %s',
        defaults: ['type spec here']
    };
}
SpriteMorph.prototype.initBlocks();

HatBlockMorph.prototype.isStop = function () {
    return ([
        'demoVariable',
        //'demoCustomBlock'        
    ].indexOf(this.selector) > -1);
};

HatBlockMorph.prototype.blockSequence = function () {
    // override my inherited method so that I am not part of my sequence
    // excluding the demo hats, so they can be executed
    var result;
    if (this.selector === 'demoVariable' || this.selector === 'demoCustomBlock'){
        result = [this];
    } else {
        result= HatBlockMorph.uber.blockSequence.call(this);
        result.shift();
    }   
    return result;
};


orSpriteMorphblockTemplates = SpriteMorph.prototype.blockTemplates;
SpriteMorph.prototype.blockTemplates = function (category){
    var blocks = orSpriteMorphblockTemplates.call(this,category),
            newBlock;
    if (category == 'variables' && world.role !== 'STUDENT'){
        newBlock = SpriteMorph.prototype.blockForSelector('demoVariable', true);        
        newBlock.isTemplate = true;        
        blocks.push(newBlock);
        newBlock = SpriteMorph.prototype.blockForSelector('demoCustomBlock', true);
        newBlock.isTemplate = true;
        blocks.push(newBlock);
    }
    
    // ////// dummy blocks ////////////
    if (world.role !== 'STUDENT' && category !== 'depository'){ 
        ['dummyCommand','dummyReporter','dummyPredicate'].forEach(
            function(spec){
                var newBlock;
                newBlock = SpriteMorph.prototype.blockForSelector(spec, true);                
                newBlock.setCategory(category || 'motion');
                newBlock.isTemplate = true;
                //fullCopy is called when a block from palette is dropped in scripts
                //In this action the block changes to the defined specification
                //in it's template input
                newBlock.fullCopy = function(){
                    var retBlock;
                    this.instantiationSpec = this.inputs()[0].evaluate();
                    retBlock = BlockMorph.prototype.fullCopy.call(this);
                    retBlock.toBlockXML = dummyBlocktoXML.bind(retBlock);
                    return retBlock;
                }
                blocks.push('-');
                blocks.push(newBlock);
               
        });
    }
    return blocks;
}

orStageMorphblockTemplates = StageMorph.prototype.blockTemplates;
StageMorph.prototype.blockTemplates = function (category){
    var blocks = orStageMorphblockTemplates.call(this,category),
            newBlock;
    if (category == 'variables' && world.role !== 'STUDENT'){
        newBlock = SpriteMorph.prototype.blockForSelector('demoVariable', true);
        newBlock.isTemplate = true;
        blocks.push(newBlock);
        newBlock = SpriteMorph.prototype.blockForSelector('demoCustomBlock', true);
        newBlock.isTemplate = true;
        blocks.push(newBlock);
    }
    
    // ////// dummy blocks ////////////
    if (world.role !== 'STUDENT' && category !== 'depository'){ 
        ['dummyCommand','dummyReporter','dummyPredicate'].forEach(
            function(spec){
                var newBlock;
                newBlock = SpriteMorph.prototype.blockForSelector(spec, true);                
                newBlock.setCategory(category || 'motion');
                newBlock.isTemplate = true;
                newBlock.fullCopy = function(){
                    var retBlock;
                    this.instantiationSpec = this.inputs()[0].evaluate();
                    retBlock = BlockMorph.prototype.fullCopy.call(this);
                    retBlock.toBlockXML = dummyBlocktoXML.bind(retBlock);
                    return retBlock;
                }               
                blocks.push('-');
                blocks.push(newBlock);
        });
    }
    return blocks;
}

// /////////////// dummy blocks ///////////////////
StageMorph.prototype.dummyCommand =
        SpriteMorph.prototype.dummyCommand = nop;
StageMorph.prototype.dummyReporter =
        SpriteMorph.prototype.dummyReporter = nop;
StageMorph.prototype.dummyPredicate =
        SpriteMorph.prototype.dummyPredicate = nop;

// --- serialization ---
function dummyBlocktoXML (serializer) {
        //do not serialize the inputs of blocks dealing with network data
    var type = this instanceof CommandBlockMorph ? 'command' :
                (this.isPredicate ? 'predicate' : 'reporter');      
   	return serializer.format(
        '<block isDummy="true" spec="@" cat="@" type="@" isDraggable="@">%%</block>',
        this.blockSpec,
        this.category,
	type,
        this.isDraggable,
        serializer.store(this.inputs()),
        this.comment ? this.comment.toXML(serializer) : ''
	);
};
// --- deserialization ---
orSnapSerializerloadBlock = SnapSerializer.prototype.loadBlock;
SnapSerializer.prototype.loadBlock = function (model, isReporter, object){
    var block,inputs;
    //if (model.tag === 'block'){
        if (Object.prototype.hasOwnProperty.call(
                model.attributes,
                'isDummy'
            )){
                switch (model.attributes.type){                    
                    case 'reporter' :
                        block = new ReporterBlockMorph();
                        break;
                    case 'predicate' :
                        block = new ReporterBlockMorph(true);
                        break;
                    default : // command
                        block = new CommandBlockMorph();
                }
                block.selector = 'dummyCommand';
                block.setSpec(model.attributes.spec);
                block.category = model.attributes.cat;
                block.setDraggable(model.attributes.isDraggable == "true");
                block.toBlockXML = dummyBlocktoXML.bind(block);
                
                inputs = block.inputs();
                model.children.forEach(function (child, i) {
                    this.loadInput(child, inputs[i], block, object);
                }, this);
                
                return block;
            }
   // }
    return orSnapSerializerloadBlock.call(this,model, isReporter, object);
}

// /////////////// dummy blocks //////////////////


// /////////////DEMO vars and custom blocks creation/////////////// 

// demoVariable block creation
SpriteMorph.prototype.demoVariable = function(spec,scope){
    var ide = this.parentThatIsA(IDE_Morph),
    categories,catButton,
    name = spec.trim(),
    palette,pButton, myself=this, dlg, stepInc,
    step1,step2,step3;

    if (name === '') {return;}
    
//go to variables category
    step1 = function(){
        categories = ide.categories.children;
        catButton = categories.find(function(element){
            return element.labelString.toUpperCase() == "variables".toUpperCase();
        });
        catButton.mouseClickLeft();
        ide.nextSteps([step2]);
    }    
        
//highlight the "make a variable button"
    
    
    step2 = function(){
        palette = ide.palette.children.find(function (element){return element instanceof FrameMorph});    
    //scroll to the top if necessary (animating)
        if (palette.parent.contents.top()<palette.parent.top()){
            ide.fps = 3;
            palette.parent.step = function (){
                if (this.contents.top()<this.top()){
                   this.scrollY(50);
                   this.adjustScrollBars();
                } else {           
                    this.step = null;
                    ide.fps = 1;
                    ide.nextSteps([step3]);
                }
            }
        } else {
            ide.fps = 1;
            ide.nextSteps([step3]);
        }   
      
        //palette.parent.scrollY(palette.parent.top()-palette.parent.contents.top()); 
        pButton = palette.children.find(function(element){
            return element instanceof PushButtonMorph && element.labelString.toUpperCase() == "make a variable".toUpperCase();
        });
        pButton.mouseEnter();
    }      
    
//show the demo dialog
    step3 = function(){
        dlg = new VariableDialogMorph(null,null,this);
        dlg.promptDemo('Variable name',this.world());
        dlg.setType(scope || 'global');
    //put the name in the text field by steping, simulating the keyboard typing
        dlg.fps = 3;
        stepInc = 1;
        dlg.step = function(){      
            if (stepInc <= name.length){
                this.body.setContents(name.substr(0,stepInc));
                stepInc++;
            } else {
                this.fps = 0;
                this.step = null;    
            }
        }
        ide.fps = 0;
    }
    
    if (ide.curreCategory!=='variables'){
        ide.nextSteps([step1]);
    } else {
        ide.nextSteps([step2]);
    }
}

StageMorph.prototype.demoVariable =
        SpriteMorph.prototype.demoVariable;

VariableDialogMorph.prototype.promptDemo = function (title,world){
    var txt = new InputFieldMorph();
       
    txt.setWidth(250);
    this.labelString = title;
    this.createLabel();

    if (!this.key) {
        this.key = 'prompt demo var' + title;
    }

    this.addBody(txt);
    txt.drawNew();
    //this.addButton('ok', 'OK');
    this.addButton('cancel', 'Cancel');
    this.fixLayout();
    this.drawNew();
    this.fixLayout();
    this.popUp(world);    
}

// demo custom block creation
SpriteMorph.prototype.demoCustomBlock = function(spec,tp,cat,scp,params){ 
var dlg,// = new BlockDialogMorph(null,null,this),
    type = tp || 'command',
    category = cat || 'motion',
    scope = scp || 'global',
    name,// = spec.trim(),
    commands,
    step1,step2,step3,step4,steps=[],
    myself=this;

//console.log(arguments.callee.caller.arguments);

    //the following hack, allows to get the actual hat block and the script attached to it
    //arguments.callee is function demoCustomBlock
    //arguments.callee.caller is Process.prototype.evaluateBlock(block, argCount)
    commands = arguments.callee.caller.arguments[0].nextBlock();

    //replace parameteter placeholders in spec (%)
    //with the corresponding params names
    name=BlockMorph.prototype.parseSpec(spec);    
    name=name.map(function(el){
            var replacement;
            if (el === '(%)'){
                replacement = params.contents.shift();
                if (replacement){
                    return '%'+replacement+' ';
                } else {
                    return '';
                }
            } else{
                return el+' ';
            }
        });
    name = ''.concat.apply('', name).trim();

    if (name === ''){
        return;
    }
    
    dlg = new BlockDialogMorph(
        null,
        function (definition) {
            var cmd,dlg,hat,scripts,repBlock;
            if (definition.spec !== '') {
                if (commands) {
                    cmd = commands.fullCopy();
                    cmd.isDraggable = commands.isDraggable; //retain the status of first block after fullcopy
                }  
                dlg = new BlockEditorMorph(definition, myself);                
                dlg.popUp();
               
                scripts = detect( // == ScriptsMorph of BlockEditorMorph
                    dlg.body.children,
                    function (el){ return el instanceof ScriptsMorph;}
                )
                //manipulate SciptsMorph
                scripts.enableKeyboard = false;
                //  override ScriptsMorph methods 

                //dropping behavior - reject drops
                scripts.wantsDropOf = function (aMorph) {
                    return false;                    
                }    
                //scripts menu
                scripts.userMenu = function(){        
                    var menu = new MenuMorph(this);
                        menu.addItem(
                            'scripts pic...',
                            'exportScriptsPicture',
                            'open a new window\nwith a picture of all scripts'
                        );
                        return menu;                    
                }

                //overriden to use different buttons
                scripts.addToolbar=hintsEditorAddToolbar.bind(scripts);
                scripts.undrop = function (){       
                    if (myself.isTeacher){
                        ScriptsMorph.prototype.undrop.call(this);
                    } else {
                        hintsUndrop.call(this);
                    }
                }
                scripts.redrop = function (){       
                    if (myself.isTeacher){
                        ScriptsMorph.prototype.redrop.call(this);
                    } else {
                        hintsRedrop.call(this);
                    }
                }
                //overide adjustbounds
                scripts.adjustBounds = adjustBounds.bind(scripts);
                
                //get rid of the old toolbar and create the new overiden one
                scripts.scrollFrame.toolBar.destroy();
                scripts.scrollFrame.toolBar=null;
                scripts.updateToolbar();
                
                dlg.scripts=scripts;
                hat = detect( 
                    dlg.scripts.children,
                    function (el){ return el instanceof PrototypeHatBlockMorph;}
                )
                //clear initial blocks, and mouse behavior
                hat.mouseClickLeft = nop;
                hat.userMenu = nop;
                repBlock = hat.nextBlock(); //repBlock is the initial report block for reporters
                if (repBlock){
                    repBlock.userDestroy(); //remove the initial report block for reporters
                    //clear drop queue
                    dlg.scripts.dropRecord = null;
                    dlg.scripts.clearDropInfo();
                    dlg.scripts.recordDrop();
                }
                
                if (cmd){
                    hat.nextBlock(cmd);
                }            
                
                hat.forAllChildren(function(child){
                    child.mouseClickLeft = nop;
                    child.userMenu = nop;
                });                   
                
                dlg.scripts.scrollFrame.nextSteps([
//put script in history                    
                    function(){
                        if (cmd){
                            HintsEditorMorph.prototype.decomposeScript.call(dlg,hat,10,7);
                            dlg.fps = 2;
                        }
                    },
//animate first step                    
                    function(){
                       this.toolBar.undoButton.mouseClickLeft();
                       dlg.fps = 0;
                   } 
                ]);
            }
        },
        this
       );
    
    dlg.prompt('Make a block',null, this.world());      
    dlg.fps=0.75;
    setButtons(false); //disable buttons
    //step 1: set the category
    if (category !== 'motion'){
        step1 = function () {
            dlg.category = category;
            dlg.categories.children.forEach(function (each) {
                each.refresh();
            dlg.types.children.forEach(function (each) {
                    each.setColor(SpriteMorph.prototype.blockColor[category]);
                });    
            });
        };
        steps.push(step1);
    }
    //step 2: set the type
    if (type !== 'command'){
        step2 = function () {
            dlg.setType(type);
        };
        steps.push(step2);
    }
    //step 3: set the scope
    if (scope !== 'global'){
        step3 = function () {
            dlg.setScope(type);
        };
        steps.push(step3);
    }
    //step 4: set the spec
    if (name.length>0){
        step4 = function(){
            var stepInc = 1;
            dlg.fps = 3;
            dlg.step = function(){      
                if (stepInc <= name.length){
                    this.body.setContents(name.substr(0,stepInc));
                    stepInc++;
                } else {
                    setButtons(true); //enable buttons, when finished
                    this.step = null;    
                }
            }
        };
         steps.push(step4);
    }
    //animate
    dlg.nextSteps(steps);
    
    //helper for enabling, disabling dialog buttons
    function setButtons(enabled){ //true is for enabling
        dlg.buttons.children.forEach(function(button){
            button.isDisabled = !enabled;
        })
    }
}
StageMorph.prototype.demoCustomBlock =
        SpriteMorph.prototype.demoCustomBlock;