// Hints editor creation,manipulation and serialization


// HintsEditorMorph ////////////////////////////////////////////////////

// HintsEditorMorph inherits from DialogBoxMorph:

HintsEditorMorph.prototype = new DialogBoxMorph();
HintsEditorMorph.prototype.constructor = HintsEditorMorph;
HintsEditorMorph.uber = DialogBoxMorph.prototype;

// HintsEditorMorph instance creation:
function HintsEditorMorph(sprite) {
    this.init(sprite);
}

HintsEditorMorph.prototype.init = function (sprite) { // target = sprite
    var tabCorner = 15,
        tabColors = IDE_Morph.prototype.tabColors,
        tabBar = new AlignmentMorph('row', -tabCorner * 2),
        tab, myself=this;  
    // additional properties:    
    this.scripts = null;
    this.handle = null;
    this.isTeacher = true;
    this.scriptsBackUp = null; //to store the original hints, in Student Mode, in order to be saved
    this.varAndCustomScripts = null;
    this.varAndCustomScriptsBackUp = null; //to store the original hints, in Student Mode, in order to be saved
    this.activeEditor = null;
    this.currentTab = 'scripts';
    this.tabBar = null;
    
    // initialize inherited properties:
    HintsEditorMorph.uber.init.call(
        this,
        sprite,
        null, //function () {myself.updateDefinition(); },
        sprite
    );

    // override inherited properites:
    this.key = 'editHints' + sprite.name;
    this.labelString = 'Hints Editor';
    this.createLabel();
    this.setPicture(new SpriteIconMorph(sprite).fullImageClassic());
    this.version = sprite.version;

    // tab bar
    tabBar.tabTo = function (tabString) {
        var active;
        if (myself.activeEditor && myself.activeEditor.contents.isAnimating){return;}
        myself.currentTab = tabString;
        this.children.forEach(function (each) {
            each.refresh();
            if (each.state) {active = each; }
        });
        active.refresh(); // needed when programmatically tabbing
        myself.switchToEditor(tabString);
        myself.fixLayout();
    };
    tab = new TabMorph(
        tabColors,
        null, // target
        function () {tabBar.tabTo('scripts'); },
        localize('Scripts'), // label
        function () {  // query
            return myself.currentTab === 'scripts';
        }
    );
    tab.padding = 3;
    tab.corner = tabCorner;
    tab.edge = 1;
    tab.labelShadowOffset = new Point(-1, -1);
    tab.labelShadowColor = tabColors[1];
    tab.labelColor = IDE_Morph.prototype.buttonLabelColor;
    tab.drawNew();
    tab.fixLayout();
    tabBar.add(tab);
    
    tab = new TabMorph(
        tabColors,
        null, // target
        function () {tabBar.tabTo('vars'); },
        localize('Vars and Custom Blocks'), // label
        function () {  // query
            return myself.currentTab === 'vars';
        }
    );
    tab.padding = 3;
    tab.corner = tabCorner;
    tab.edge = 1;
    tab.labelShadowOffset = new Point(-1, -1);
    tab.labelShadowColor = tabColors[1];
    tab.labelColor = IDE_Morph.prototype.buttonLabelColor;
    tab.drawNew();
    tab.fixLayout();
    tabBar.add(tab);

    tabBar.fixLayout();
    tabBar.children.forEach(function (each) {
        each.refresh();
    });
    this.tabBar = tabBar;
    this.add(this.tabBar);


    // create scripting areas    
    this.createScriptsEditor();
    this.createVarAndCustomScriptsEditor();
    this.tabBar.tabTo('scripts');

    this.addButton('ok', 'OK'); 

    this.setExtent(new Point(450, 350)); // normal initial extent
    this.fixLayout();
    
};

HintsEditorMorph.prototype.ok = function(){
    if (this.isTeacher){ //clean up, before closing
        this.scripts.cleanUp();
        this.varAndCustomScripts.cleanUp();
    }
    this.accept();
}


//main scripts editor
HintsEditorMorph.prototype.createScriptsEditor = function(){
    var myself = this;
    if (!this.scripts){
        this.scripts = new ScriptsMorph();
    }    
    this.scripts.rejectsHats = false;  
    this.scripts.isDraggable = false;
    this.scripts.color = IDE_Morph.prototype.groupColor;
    this.scripts.cachedTexture = IDE_Morph.prototype.scriptsPaneTexture;
    this.scripts.cleanUpMargin = 10;
    this.scripts.enableKeyboard = false;
    this.scripts.fixMultiArgs();
//  override ScriptsMorph methods  
    this.scripts.scriptTarget = function(){
        return myself.target;
    }
    
    //hints scripts dropping behavior
    this.scripts.wantsDropOf = function (aMorph) {
        if (myself.isTeacher &&
                (!aMorph.selector ||
                    (aMorph.selector &&
                        (aMorph.selector !== 'demoVariable' && aMorph.selector !== 'demoCustomBlock')
                    )
                )              
            ){
            return ScriptsMorph.prototype.wantsDropOf.call(this,aMorph);
        } else {
            return false;
        }
    }    
    //hints scripts menu
    this.scripts.userMenu = function(){        
        var menu;   
        if (myself.isTeacher){
            return ScriptsMorph.prototype.userMenu.call(this);
        } else {
            menu = new MenuMorph(this);
            menu.addItem(
                'scripts pic...',
                'exportScriptsPicture',
                'open a new window\nwith a picture of all scripts'
            );
            return menu;
        }
    }       
    
    //overriden to use different buttons
    this.scripts.addToolbar = function (){       
        if (myself.isTeacher){
            return ScriptsMorph.prototype.addToolbar.call(this);
        } else {
            return hintsEditorAddToolbar.call(this);
        }
    }
    this.scripts.undrop = function (){       
        if (myself.isTeacher){
            ScriptsMorph.prototype.undrop.call(this);
        } else {
            hintsUndrop.call(this);
        }
    }
    this.scripts.redrop = function (){       
        if (myself.isTeacher){
            ScriptsMorph.prototype.redrop.call(this);
        } else {
            hintsRedrop.call(this);
        }
    }
    //overide adjustbounds
    this.scripts.adjustBounds = adjustBounds.bind(this.scripts);    

}

//vars and custom blocks editor
HintsEditorMorph.prototype.createVarAndCustomScriptsEditor = function(){
    var myself = this;
    if (!this.varAndCustomScripts){
        this.varAndCustomScripts = new ScriptsMorph();
    }    
    this.varAndCustomScripts.rejectsHats = false;  
    this.varAndCustomScripts.isDraggable = false;
    this.varAndCustomScripts.color = IDE_Morph.prototype.groupColor;
    this.varAndCustomScripts.cachedTexture = IDE_Morph.prototype.scriptsPaneTexture;
    this.varAndCustomScripts.cleanUpMargin = 10;
    this.varAndCustomScripts.enableKeyboard = false;
    this.varAndCustomScripts.fixMultiArgs();
//  override ScriptsMorph methods  
    this.varAndCustomScripts.scriptTarget = function(){
        return myself.target;
    }
    
    //hints scripts dropping behavior
    
    this.varAndCustomScripts.wantsDropOf = function (aMorph) {                  
        if (myself.isTeacher && aMorph instanceof BlockMorph) {
            return ScriptsMorph.prototype.wantsDropOf.call(this,aMorph);
        } else {
            return false;
        }
    }
    this.varAndCustomScripts.reactToDropOf = function (droppedMorph, hand) {
        var target; 
        if (droppedMorph instanceof BlockMorph)  {
                if (droppedMorph instanceof CommandBlockMorph){
                    target = droppedMorph.closestAttachTarget();
                    if (target){
                        target = target.element;
                    }
                } else {
                    target = this.closestInput(droppedMorph,hand);
                }    
                if (droppedMorph.selector == 'demoVariable' ||
                    droppedMorph.selector == 'demoCustomBlock' ||
                    (target && target.topBlock().selector == 'demoCustomBlock')
                    ){
                        droppedMorph.snap(hand);
                    } else {
                        droppedMorph.slideBackTo(hand.grabOrigin);
                    }
        }
        this.adjustBounds();
    };
    
    
    //hints scripts menu
    this.varAndCustomScripts.userMenu = function(){        
        var menu;   
        if (myself.isTeacher){
            //return ScriptsMorph.prototype.userMenu.call(this);
            menu = new MenuMorph(this);
            menu.addItem('clean up', 'cleanUp', 'arrange scripts\nvertically');    
            menu.addItem(
                'scripts pic...',
                'exportScriptsPicture',
                'open a new window\nwith a picture of all scripts'
            );
            return menu;
        } else {
            return null;            
        }
    }       
    /*
    //overriden to use different buttons
    this.varAndCustomScripts.updateToolbar = function (){       
        if (myself.isTeacher){
            return ScriptsMorph.prototype.updateToolbar.call(this);
        } else { //no toolbar in STUDENT mode
            if (this.toolBar){
                this.toolBar.destroy();
            }    
        }
    }
    */
   //overide adjustbounds
    this.varAndCustomScripts.adjustBounds = adjustBounds.bind(this.varAndCustomScripts);
}

//Switch between scripts and vars&customBlocks editors
HintsEditorMorph.prototype.switchToEditor = function(editor){
    /*
    //restores the position of script blocks, when switching
    function rearrangeScriptsPositions(){
        var pos = this.position();
        this.children.forEach(function (child) {
            if (!child.block) { // omit anchored comments
                child.setPosition(child.position().subtract(pos));
                if (child instanceof BlockMorph) {
                    child.allComments().forEach(function (comment) {
                        comment.align(child);
                    });
                }
            }
        })
    }
    */
    if (this.activeEditor) {
        this.activeEditor.destroy();
    }
    if (editor == 'scripts'){
        this.activeEditor = new ScrollFrameMorph(this.scripts);
        //rearrangeScriptsPositions.call(this.scripts);
    } else {
        this.activeEditor = new ScrollFrameMorph(this.varAndCustomScripts);
        //rearrangeScriptsPositions.call(this.varAndCustomScripts);
    }
    this.activeEditor.padding = 10;
    this.activeEditor.growth = 50;
    this.activeEditor.isDraggable = false;
    this.activeEditor.acceptsDrops = false;
    this.activeEditor.contents.acceptsDrops = this.isTeacher;
    this.activeEditor.contents.adjustBounds();
    if (editor == 'scripts'){
        this.scripts.scrollFrame = this.activeEditor;
        this.scripts.updateToolbar();
    } else {
        this.varAndCustomScripts.scrollFrame = this.activeEditor;        
        this.varAndCustomScripts.updateToolbar();
    }    

    this.addBody(this.activeEditor);
}

HintsEditorMorph.prototype.popUp = function () {
    var world = this.target.world(),
        ide = this.target.parentThatIsA(IDE_Morph);
    
    if (this.version !== this.target.version){
        this.setPicture(new SpriteIconMorph(this.target).fullImageClassic());
        this.fixLayout();
    }
    
    if (world) {
        if (ide){ //stop all scripts before poping
            ide.stopAllScripts();
        }
        HintsEditorMorph.uber.popUp.call(this, world);
        if (!this.handle) {//this.handle.destroy();}
        this.handle = new HandleMorph(
            this,
            280,
            220,
            this.corner,
            this.corner
        );}
        world.keyboardReceiver = null;
        this.activeEditor.contents.adjustBounds();
    }
};

HintsEditorMorph.prototype.fixLayout = function () {
    var th = fontHeight(this.titleFontSize) + this.titlePadding * 2;

    if (this.buttons && (this.buttons.children.length > 0)) {
        this.buttons.fixLayout();
    }

    if (this.head) {       
        this.head.setPosition(this.position().add(new Point(
            (this.width()- this.head.width())/ 2 + this.padding,
            th + this.padding
        )));    
    }

    if (this.body) {
        if (this.head) {
            this.body.setLeft(this.left()+this.padding);
            this.body.setTop(this.head.bottom()+this.padding *2);
            this.body.setExtent(new Point(
                this.width() - this.padding * 2,
                this.height() - this.padding * 5 - th - this.head.height() - this.buttons.height()
            ));                   
           
        } else {
            this.body.setPosition(this.position().add(new Point(
                this.padding,
                th + this.padding
            )));
            this.body.setExtent(new Point(
                this.width() - this.padding * 2,
                this.height() - this.padding * 3 - th - this.buttons.height() 
            ));
        }
        this.tabBar.setLeft(this.body.left());
        this.tabBar.setBottom(this.body.top());
    }

    if (this.label) {
        this.label.setCenter(this.center());
        this.label.setTop(this.top() + (th - this.label.height()) / 2);
    }

    if (this.buttons && (this.buttons.children.length > 0)) {
        this.buttons.setCenter(this.center());
        this.buttons.setTop(this.body.bottom() + this.padding);
    }
  
};


/*
HintsEditorMorph.prototype.decomposeScript-old = function (bScript,x,y){
    var nestedBlocks,
        scriptPos = new Point(+x,+y),
        start,record, slotParent,
        myself = this;
    
    //put all block in script, to an array
    nestedBlocks = bScript.allChildren().filter(
            function (child) {
                return (child instanceof BlockMorph 
                        && !child.isTemplate);
            }
    );   
    // store the position of the top block, in ordet to calculate the positions of the others
    start = nestedBlocks[0].position();
    
    //create the necessary information for each block, in order to put it in drop history
    //block are scanned in reverse, in order to retain their original position in the script
    nestedBlocks = nestedBlocks.reverse().map(function (e){
        var position,nb,lastTarget,lastInput, oldMenu;
        
        // calculate the position, relative to the top block
        position = e.position().subtract(start);     
        if (!e.isDraggable){            
            e.isPreserved = true;
        }
        //command blocks -- remove next block, make it single
        if (e instanceof CommandBlockMorph){
            nb = e.nextBlock();
            if (nb){
                e.children.splice(e.children.indexOf(nb),1); // remove next block, make block single
            }
        }
        
        //remove CSlot contents, if any -- clean command slot contents
        e.forAllChildren(function(child){
            var nstb;
            if ((child instanceof CSlotMorph) ||(child instanceof CommandSlotMorph)){ 
               nstd = child.nestedBlock();
               if (nstd){
                   child.children.splice(0,1);
                   child.fixLayout();
               }
            } 
        });
        
        //reporter blocks
        //Due to an original snap bug, reporter block have to keep track of the
        //input they replace in their parent block, in order to redrop correctly
        //snap, prepareToBeGrabbed(reporter blocks) and revertToDefault of parent methods
        //have to be overriden
        
        lastTarget = null;
        lastInput = null;
        if (e instanceof ReporterBlockMorph){
            if (e.parent){
                lastTarget = e.parent;
                lastInput = revertToDefaultInput(lastTarget,e,true); //revert to default, and answer the replaced input
                e.parent.fixLayout();
                e.lastInputSlot = lastInput;
                e.snap = snap.bind(e); //overide snap method
                e.prepareToBeGrabbed = prepareToBeGrabbed.bind(e); //overide prepareToBeGrabbed method   
            }
            if (e.parent instanceof RingReporterSlotMorph){
                //if it's a variable inside a ring, overide method, so the ring does not disappear
                e.parent.replaceInput = RingReporterSlotMorph.uber.replaceInput;
            }
        }
        e.isDraggable = false;
        if (!e.parent){ //it case of top blocks
            e.parent = myself.scripts.scrollFrame;
        }    
        //overide user menu
        oldMenu = e.userMenu;
        e.userMenu = function (){
            if (myself.isTeacher){
                return oldMenu.call(this);
            } else {
                if ((e instanceof CustomCommandBlockMorph) || (e instanceof CustomReporterBlockMorph)){
                    return BlockMorph.prototype.userMenu.call(this);
                } else {
                    return oldMenu.call(this);
                }            
            }
        }
        e.mouseClickLeft = nop;        
        //store the block information
        return {block:e, position:scriptPos.add(position)};//, lastTarget:lastTarget, lastInput:lastInput};
    })
           
    //create the drop history, bottom up
    for (var i=0; i<nestedBlocks.length; i++){   
        if (!nestedBlocks[i].block.isPreserved){    
            record = {
                    lastDroppedBlock: nestedBlocks[i].block,
                    lastReplacedInput: null,//nestedBlocks[i].lastInput,
                    lastDropTarget: null,//nestedBlocks[i].lastInput,
                    lastPreservedBlocks: null,
                    lastNextBlock: null,
                    lastWrapParent: null,
                    lastOrigin: {origin:this.scripts, position:nestedBlocks[i].position},
                    action: 'delete',
                    situation: null,
                    lastRecord: this.scripts.dropRecord,
                    nextRecord: null
                };        
            this.scripts.dropRecord=record;
            this.scripts.dropRecord.lastRecord.nextRecord = record;
        } else {                     
            if (!(nestedBlocks[i].block.parent instanceof SyntaxElementMorph)){
                nestedBlocks[i].block.setPosition(nestedBlocks[i].position.add(this.scripts.topLeft()));
                this.scripts.add(nestedBlocks[i].block);
            } else {
                if (!nestedBlocks[i].block.parent.isDraggable){
                    nestedBlocks[i].block.parent.add(nestedBlocks[i].block);
console.log('1',nestedBlocks[i].block);                    
                } else { //it's nested
console.log('2',nestedBlocks[i].block);                   
                    if (nestedBlocks[i].block instanceof CommandBlockMorph){
                        slotParent = nestedBlocks[i].block.parentThatIsA(CSlotMorph);
console.log('3',slotParent);                
                        if (slotParent && !slotParent.parent.isDraggable){
                            slotParent.add(nestedBlocks[i].block);
                            slotParent.fixLayout();
                        }    
                    }    
                }
            }    
        }  
    }
    this.scripts.updateToolbar();//   
}
*/
HintsEditorMorph.prototype.decomposeScript = function (bScript,x,y){
    var nestedBlocks, previousStep, myscripts=this.scripts;
    
    nestedBlocks = bScript.allChildren().filter(
            function (child) {
                // overide the attachTargets behavior, not to support nested autowrapping
                if (child instanceof CommandBlockMorph){                   
                    child.attachTargets = attachTargets.bind(child);
                }
                // and return only the undraggables
                return (child instanceof BlockMorph 
                          && !child.isTemplate && child.isDraggable);
                }
        );
    
    bScript.setPosition(new Point(x,y).add(this.scripts.bounds.origin));    
    this.scripts.add(bScript);
    //store previous step, if any
    previousStep=this.scripts.onNextStep;
    //so that it executes next time. This way we support multiple scripts
    this.scripts.onNextStep = function (){            
            nestedBlocks.reverse().forEach(function (block){
                var hand;
                if (block instanceof ReporterBlockMorph){
                    if (block.parent && block.parent.inputs){
                        block.snap = snap.bind(block);
                        block.lastInputIndex=block.parent.inputs().indexOf(block);
                        block.lastParent = block.parent; //in case of multiarg input
                    }
                    block.isDraggable = false;
                    block.userDestroy();
                } else {
                    hand = new Morph();
                    if (block.parent && block.parent instanceof CommandBlockMorph){                       
                        hand.setPosition(block.parent.position());
                    } else {                      
                        hand.setPosition(block.position());
                    }                            
                    block.isDraggable = false;
                    block.userDestroy();
                }               
                
            });
            if (previousStep){
                previousStep.call(myscripts);
            }
        };  
    
    
}
//overidden snap for reporters
function snap(hand){
    // passing the hand is optional (for when blocks are dragged & dropped)
    var scripts = this.parent,
        nb, myself=this,
        target, targetBlock;

    this.cachedSlotSpec = null;
    if (!(scripts instanceof ScriptsMorph)) {
        return null;
    }
    scripts.clearDropInfo();
    scripts.lastDroppedBlock = this;

    //drop target in the input slot that was originally replaced and not the(default)
    // lastInputIndex == the index of the parent's inputs, where the reporter previously was
    // scripts.topMorphAt(this.position()) is the parent, already dropped back
    //closest input(Snap bug)      
   
//    targetBlock=scripts.topMorphAt(this.position());
    /*if (!(targetBlock instanceof BlockMorph)){
        targetBlock = targetBlock.parentThatIsA(BlockMorph);
    }*/
    /*if (targetBlock !== this.lastParent){
        targetBlock = detect(
                targetBlock.children,
                function (child){
                    return child === myself.lastParent;
                }
            );
    }
    if (!isNil(targetBlock)){        
            target = targetBlock.inputs()[this.lastInputIndex];          
    }
    */
    target = this.lastParent.inputs()[this.lastInputIndex]; 
    if (!isNil(target)) {
        scripts.lastReplacedInput = target;
        scripts.lastDropTarget = target.parent;
        if (target instanceof MultiArgMorph) {
            scripts.lastPreservedBlocks = target.inputs();
            scripts.lastReplacedInput = target.fullCopy();
        } else if (target instanceof CommandSlotMorph) {
            scripts.lastReplacedInput = target;
            nb = target.nestedBlock();
            if (nb) {
                nb = nb.fullCopy();
                scripts.add(nb);               
                nb.moveBy(nb.extent());
                nb.fixBlockColor();
                scripts.lastPreservedBlocks = [nb];
            }
        }
        target.parent.replaceInput(target, this);
        this.lastInputSlot = null;
        if (this.snapSound) {
            this.snapSound.play();
        }
    }
    this.startLayout();
    this.fixBlockColor();
    this.endLayout();
    ReporterBlockMorph.uber.snap.call(this);
}
/*
//overidden so it returns the replaced input slot 
function revertToDefaultInput(target, arg, noValues){
   
    var idx = target.parts().indexOf(arg),
        inp = target.inputs().indexOf(arg),
        deflt = new InputSlotMorph(),
        def;

    if (idx !== -1) {
        if (target instanceof BlockMorph) {
            deflt = target.labelPart(target.parseSpec(target.blockSpec)[idx]);
            if (target.isCustomBlock) {
                def = target.isGlobal ? target.definition
                        : target.scriptTarget().getMethod(target.blockSpec);
                if (deflt instanceof InputSlotMorph) {
                    deflt.setChoices.apply(
                        deflt,
                        def.inputOptionsOfIdx(inp)
                    );
                }
                if (deflt instanceof InputSlotMorph ||
                    (deflt instanceof BooleanSlotMorph)
                ) {
                    deflt.setContents(
                        def.defaultValueOfInputIdx(inp)
                    );
                }
            }
        } else if (target instanceof MultiArgMorph) {
            deflt = target.labelPart(target.slotSpec);
        } else if (target instanceof ReporterSlotMorph) {
            deflt = target.emptySlot();
        }
    }
    // set default value
    if (!noValues) {
        if (inp !== -1) {
            if (deflt instanceof MultiArgMorph) {
                deflt.setContents(target.defaults);
                deflt.defaults = target.defaults;
            } else if (!isNil(target.defaults[inp])) {
                deflt.setContents(target.defaults[inp]);
            }
        }
    }
    target.silentReplaceInput(arg, deflt);
    if (deflt instanceof MultiArgMorph) {
        deflt.refresh();
    } else if (deflt instanceof RingMorph) {
        deflt.fixBlockColor();
    }
    target.cachedInputs = null;

    //return replaced input
    return deflt;
}

//overidden, to use the overidden revertToDefaultInputs
function prepareToBeGrabbed(handMorph) {
    var oldPos = this.position(), scripts = this.parentThatIsA(ScriptsMorph);
    if (this.lastInputSlot){return;} //if has already(from decomposescript), do nothing
    nop(handMorph);
    if ((this.parent instanceof BlockMorph)
            || (this.parent instanceof MultiArgMorph)
            || (this.parent instanceof ReporterSlotMorph)) {         
        this.lastInputSlot=revertToDefaultInput(this.parent, this,true);
        this.parent.fixLayout();
        scripts.changed();      
        this.setPosition(oldPos);
    }
    //ReporterBlockMorph.uber.prepareToBeGrabbed.call(this, handMorph);
    this.alpha = 0.85;
    this.cachedSlotSpec = null;
};
*/
//overidden
function hintsUndrop() {    
    var myself = this,
        parent = this.parentThatIsA(DialogBoxMorph),
        blockPosition,blockHeight;
        
    if (this.isAnimating) {return; }
    if (!this.dropRecord || !this.dropRecord.lastRecord) {return; }
    if (!this.dropRecord.situation) {
        this.dropRecord.situation =
            this.dropRecord.lastDroppedBlock.situation();
    }

    this.isAnimating = true;
    //prevent the HintsEditor from dragging and scrolling during animation
    parent.isDraggable = false;
    this.scrollFrame.scrollX = nop;
    this.scrollFrame.scrollY = nop;
    this.scrollFrame.vBar.mouseDownLeft = nop;
    this.scrollFrame.hBar.mouseDownLeft = nop;
    blockPosition = this.dropRecord.lastOrigin.origin.top() + this.dropRecord.lastOrigin.position.y;
    blockHeight = this.dropRecord.lastDroppedBlock.height();
    this.dropRecord.lastDroppedBlock.slideBackTo(
        this.dropRecord.lastOrigin,
        500, //tune easing time
        this.recoverLastDrop(),
        function () {
            myself.updateToolbar();
            myself.isAnimating = false;
            parent.isDraggable = true;
            myself.scrollFrame.scrollX = ScrollFrameMorph.prototype.scrollX;
            myself.scrollFrame.scrollY = ScrollFrameMorph.prototype.scrollY;
            myself.scrollFrame.vBar.mouseDownLeft = SliderMorph.prototype.mouseDownLeft;
            myself.scrollFrame.hBar.mouseDownLeft = SliderMorph.prototype.mouseDownLeft;
            myself.scrollFrame.nextSteps([
                function(){
                    var scrollTop = myself.scrollFrame.top(),
                        scrollBottom  = myself.scrollFrame.bottom();
                    //scroll so that the dropped block is visible    
                    if (blockPosition - blockHeight < scrollTop){
                        myself.scrollFrame.scrollY(scrollTop+blockHeight+10-blockPosition);
                    } else if (blockPosition + blockHeight > scrollBottom){
                        myself.scrollFrame.scrollY(scrollBottom-blockPosition-blockHeight-10);
                    }   
                }
            ])
        }
    );   
    this.dropRecord = this.dropRecord.lastRecord;
};

function hintsRedrop() {
    var inSlot;       
    if (this.isAnimating) {return; }
    if (!this.dropRecord || !this.dropRecord.nextRecord) {return; }

    this.dropRecord = this.dropRecord.nextRecord;
    if (this.dropRecord.action === 'delete') {
        this.recoverLastDrop(true);         
        inSlot = this.dropRecord.lastDroppedBlock.parentThatIsA(CSlotMorph) ||
                 this.dropRecord.lastDroppedBlock.parentThatIsA(CommandSlotMorph);         
        this.dropRecord.lastDroppedBlock.destroy();
        if (inSlot){
            inSlot.fixLayout();
            this.changed();
        }
        if (this.dropRecord.lastDroppedBlock instanceof ReporterBlockMorph){
            this.changed();
        }
        this.updateToolbar();
    }
    
};





//load serialized scripts in STUDENT hints script history
SnapSerializer.prototype.loadScriptsInHistory = function (object, scripts, model) {
    var myself = this,
        scale = SyntaxElementMorph.prototype.scale;
    scripts.cachedTexture = IDE_Morph.prototype.scriptsPaneTexture;
    model.children.forEach(function (child) {
        var element, position;
        if (child.tag === 'script') {
            element = myself.loadScript(child, object);
            if (!element) {
                return;
            }
            position =new Point(
                (+child.attributes.x || 0) * scale,
                (+child.attributes.y || 0) * scale
            );
            element.fixBlockColor(null, true); // force zebra coloring
            object.decomposeScript(element,position.x,position.y);           
            /*element.allComments().forEach(function (comment) {
                comment.align(element);
            });*/
        } else if (child.tag === 'comment') {
            element = myself.loadComment(child);
            if (!element) {
                return;
            }
            element.setPosition(new Point(
                (+child.attributes.x || 0) * scale,
                (+child.attributes.y || 0) * scale
            ).add(scripts.topLeft()));
            element.isDraggable = false;
            element.contents.isEditable = false;
            scripts.add(element);
        }
    });
};

//load serialized VARS and CUSTOM BLOCKs scripts 
//in STUDENT "vars and blocks" script as buttons
SnapSerializer.prototype.loadScriptsAsButtons = function (object, scripts, model) {
    var myself = this,
        scale = SyntaxElementMorph.prototype.scale,
        varsLabel, varsMaxWidth, varsNextY,
        blocksLabel, blocksNextY,blockButtons=[];
    scripts.cachedTexture = IDE_Morph.prototype.scriptsPaneTexture;
    
    varsLabel = new StringMorph(localize('Variables'),16);
    varsLabel.isBold = true;
    varsLabel.color = IDE_Morph.prototype.buttonLabelColor;
    varsLabel.drawNew();
    varsLabel.setPosition(new Point(10,10));
    varsMaxWidth = varsLabel.width() + 10; //used to position the blocks
    varsNextY = varsLabel.height() + 10;   //used to position the next button
    scripts.add(varsLabel);
    
    blocksLabel = new StringMorph(localize('custom blocks'),16);
    blocksLabel.isBold = true;
    blocksLabel.color = IDE_Morph.prototype.buttonLabelColor;
    blocksLabel.drawNew();
    blocksLabel.setPosition(new Point(varsMaxWidth+10, 10));
    blocksNextY = blocksLabel.height() + 10;   //used to position the next button
    scripts.add(blocksLabel);
    
    
    model.children.forEach(function (child) {
        var element, position, button;
        if (child.tag === 'script') {
            element = myself.loadScript(child, object);
            if (!element) {
                return;
            }
            if (element.selector === 'demoVariable' && element.inputs()[0].evaluate().trim() !== ''){
                button = createVarButton(element);
                button.setPosition(new Point(10,varsNextY + 10));
                varsNextY = button.bottom() + 3;
                if (button.right()>varsMaxWidth){
                    varsMaxWidth = button.right();
                }
                scripts.add(button);
            }
            if (element.selector === 'demoCustomBlock' && element.inputs()[0].evaluate().trim() !== ''){
                button = createBlockButton(element);
                button.setPosition(new Point(10,blocksNextY + 10));
                blocksNextY = button.bottom() + 3;                
                scripts.add(button);
                blockButtons.push(button);
            }
        } 
    });
    
    //fix blocks position
    if (blockButtons.length>0){
        blocksLabel.setLeft(varsMaxWidth+10);
        blockButtons.forEach(function (b){
            b.setLeft(varsMaxWidth+10);
        });
    }
    
    //helper for creating vars and blocks buttons
    function createVarButton(block){
        var button;
        
        button = new ReporterBlockMorph();
        button.isDraggable = true; //temporary make it draggable, to have the correct label color
        button.setScale(1.4);
        button.setSpec(block.inputs()[0].evaluate());
        button.setCategory('variables');
        button.mouseClickLeft = function () {
                                    block.receiver = function () { return object.target} // palette has not been created yet. Overide the receiver to current sprite in order to execute the block
                                    object.target.parentThatIsA(StageMorph).threads.toggleProcess(block, object.target);
				}
        button.fixLayout();                        
        button.isDraggable = false; //restore, undraggable state                       
        return button;                    
    }
    
    function createBlockButton(block){
        var button;        
        switch (block.inputs()[1].evaluate()){
            case 'reporter' :
                button = new ReporterBlockMorph();
                break;
            case 'predicate' :
                button = new ReporterBlockMorph(true);
                break;
            default:
                button = new CommandBlockMorph();
        }
        
        button.isDraggable = true; //temporary make it draggable, to have the correct label color
        button.setScale(1.4);
        button.setSpec(block.inputs()[0].evaluate().replace(/\(%\)/g,'%n'));
        button.setCategory(block.inputs()[2].evaluate());
        button.mouseClickLeft = function () {
                                    block.receiver = function () { return object.target} // palette has not been created yet. Overide the receiver to current sprite in order to execute the block
                                    object.target.parentThatIsA(StageMorph).threads.toggleProcess(block, object.target);
				}
        button.fixLayout();                        
        button.isDraggable = false; //restore, undraggable state        
        return button;
    }
    
};



//do not allow comments to attach to blocks in Hints Editor
OriginalCommentMorpSnap = CommentMorph.prototype.snap;
CommentMorph.prototype.snap = function (hand) {
    if (this.parentThatIsA(HintsEditorMorph)){
        return;
    } else {
        OriginalCommentMorpSnap.call(this,hand);
    }    
}

//Hints Editor toolbar drawing
SymbolMorph.prototype.names.push('pointLeft');

originalSymbolCanvasColored  = SymbolMorph.prototype.symbolCanvasColored; 
SymbolMorph.prototype.symbolCanvasColored = function (aColor) {
    var canvas;
    if (this.name=='pointLeft'){
        canvas = newCanvas(new Point(this.symbolWidth(), this.size));
        return this.drawSymbolPointLeft(canvas, aColor);
    } else {
        return originalSymbolCanvasColored.call(this,aColor);
    }
}

originalSymbolWidth = SymbolMorph.prototype.symbolWidth;
SymbolMorph.prototype.symbolWidth = function () {
    if (this.name=='pointLeft'){
        return Math.sqrt(this.size * this.size - Math.pow(this.size / 2, 2));
    } else {
        return originalSymbolWidth.call(this);
    }
}

SymbolMorph.prototype.drawSymbolPointLeft = function (canvas, color) {
    // answer a canvas showing a left-pointing, equilateral triangle
    var ctx = canvas.getContext('2d');

    ctx.fillStyle = color.toString();
    ctx.beginPath();
    ctx.moveTo(0, Math.round(canvas.height / 2));
    ctx.lineTo(canvas.width, 0);
    ctx.lineTo(canvas.width, canvas.height);
    ctx.lineTo(0, Math.round(canvas.height / 2));
    ctx.closePath();
    ctx.fill();
    return canvas;
};

//overriden to use new buttons in student mode
function hintsEditorAddToolbar() {
    var toolBar = new AlignmentMorph(),
    	myself = this,
        shade = new Color(140, 140, 140);

    toolBar.respectHiddens = true;
    toolBar.redoButton = new PushButtonMorph(
        this,
        "redrop",
        new SymbolMorph("pointLeft", 12)
    );
    toolBar.redoButton.alpha = 0.2;
    toolBar.redoButton.padding = 2;
    // toolBar.redoButton.hint = 'redo the last undone\nblock drop\nin this pane';
    toolBar.redoButton.labelShadowColor = shade;
    toolBar.redoButton.drawNew();
    toolBar.redoButton.fixLayout();
    toolBar.add(toolBar.redoButton);
    toolBar.undoButton = new PushButtonMorph(
        this,
        "undrop",
        new SymbolMorph("pointRight", 12)
    );
    toolBar.undoButton.alpha = 0.2;
    toolBar.undoButton.padding = 2;
    // toolBar.undoButton.hint = 'undo the last\nblock drop\nin this pane';
    toolBar.undoButton.labelShadowColor = shade;
    toolBar.undoButton.drawNew();
    toolBar.undoButton.fixLayout();
    toolBar.add(toolBar.undoButton);

    //just to be compatible with updateToolbar
    toolBar.keyboardButton = new ToggleButtonMorph();

    return toolBar;
};


//overriden to adjust bounds only to the right and down, and not to the left or up
function adjustBounds() {
    var subBounds,
        newBounds,
        myself = this;
    if (this.scrollFrame === null) {
        return null;
    }   
    subBounds = this.submorphBounds(); 
    if (subBounds && (!this.scrollFrame.isTextLineWrapping)) {
            newBounds = subBounds
                .expandBy(this.scrollFrame.padding)
                .growBy(this.scrollFrame.growth)
                .merge(this.scrollFrame.bounds);
    } else {
        newBounds = this.scrollFrame.bounds.copy();
    }
 
    if (!this.bounds.eq(newBounds)) { 
        //this.bounds = newBounds;
        this.bounds = newBounds.translateBy( //keep the previous scripts origin, or else ... 
                new Point(this.bounds.origin.x - newBounds.origin.x,
                this.bounds.origin.y - newBounds.origin.y));
        this.drawNew();
        this.keepInScrollFrame();
    }

    if (this.scrollFrame.isTextLineWrapping) {
        this.children.forEach(function (morph) {
            if (morph instanceof TextMorph) {
                morph.setWidth(myself.width());
                myself.setHeight(
                    Math.max(morph.height(), myself.scrollFrame.height())
                );
            }
        });
    }

    this.scrollFrame.adjustScrollBars(); 
};

//overriden not to supprort nested auto-wrapping
function attachTargets() {
    var answer = [],
        tp;

    if (!(this instanceof HatBlockMorph)) {
        tp = this.topAttachPoint();
        if (!(this.parent instanceof SyntaxElementMorph)) {
            answer.push({
                point: tp,
                element: this,
                loc: 'top',
                type: 'block'
            });
        }
//        if (ScriptsMorph.prototype.enableNestedAutoWrapping ||
          if (!this.parentThatIsA(CommandSlotMorph)) {
            answer.push({
                point: tp,
                element: this,
                loc: 'wrap',
                type: 'block'
            });
        }
    }
    if (!this.isStop()) {
        answer.push({
            point: this.bottomAttachPoint(),
            element: this,
            loc: 'bottom',
            type: 'block'
        });
    }
    return answer;
};