var NetworkClient ={};

NetworkClient.serverAddress = "";
NetworkClient.nickName = "";
NetworkClient.io = null;
NetworkClient.users = {};
NetworkClient.sharedVariables = {};
//NetworkClient.IDE = null;

NetworkClient.connect = function (ide) {
	NetworkClient.io = io.connect(NetworkClient.serverAddress+":3333",{reconnection: false});
	//NetworkClient.IDE = ide;

	NetworkClient.io.on("connect", function(){
		ide.inform("Success","Connected to Network");
		NetworkClient.io.emit("send_nickname",NetworkClient.nickName);
	});	
	
	NetworkClient.io.on("connected_users",function(users_data){ //receive already connected users information, at the begining of connection
		NetworkClient.users = users_data;
		//console.log("initial users ", NetworkClient.users )
	});
	
	
	NetworkClient.io.on("new_user",function(user_data){ //receive user data of a new user
		NetworkClient.users[user_data.id] = {"nickname": user_data.nickname, "variables" : {} }; //store connected user	
		ide.inform("New User",user_data.nickname + " has joined the network");
		//console.log(NetworkClient.users);
	});
	
	NetworkClient.io.on("disconnected_user",function(userId){	//remove disconnected user data			
		ide.inform("User Offline" , NetworkClient.users[userId].nickname + " has disconnected!");
		delete NetworkClient.users[userId];
		//console.log("remaining users ", NetworkClient.users );
	});
	
	NetworkClient.io.on("connect_error",function(){				
		ide.inform("Error","Could not connect to server");
	});
	
	NetworkClient.io.on("disconnect",function(){				
		var globalVarFrame = ide.stage.variables.parentFrame;
		ide.inform("Warning","Disconnected from Network");
		/*for (v in globalVarFrame.vars){ //unshare all shared variables
			globalVarFrame.vars[v].isShared = false;
		}*/
                NetworkClient.sharedVariables={};
		NetworkClient.users = {};
	});
	
	//handle received sprites
	NetworkClient.io.on("receive_sprite", function(spriteData){
		ide.confirm("Sprite <"+spriteData.sprite.name+"> received from user "+ spriteData.nickName.toUpperCase() +".\n Do you want to import it?", "Import sprite confirmation", function(){
			ide.openSpritesString(spriteData.sprite.data);
		}); 
	});
	
	NetworkClient.io.on("message", function(message){
		//broadcast the message to application
		var hats =[];
        ide.stage.children.concat(ide.stage).forEach(function (morph) {
            if (morph instanceof SpriteMorph || morph instanceof StageMorph) {
                hats = hats.concat(morph.allHatBlocksFor(message));
            }
        });
        hats.forEach(function (block) {
            ide.stage.threads.processes.push(ide.stage.threads.startProcess(block, ide.stage.isThreadSafe));
        });
	});
	
	//handle received variables
	NetworkClient.io.on("variable_shared", function(data){
		NetworkClient.users[data.id]["variables"][data.varData.name] = data.varData.value; //update the connected users data
		//console.log(NetworkClient.users);
	});
	
	NetworkClient.io.on("variable_unshared", function(data){
		delete NetworkClient.users[data.id]["variables"][data.varName]; //update the connected users data
		//console.log(NetworkClient.users);
	});
	
		
}




IDE_Morph.prototype.emitSprite = function(sprite){
	var spriteStr = this.serializer.serialize(sprite.allParts());
    spriteStr = '<sprites app="'
        + this.serializer.app
        + '" version="'
        + this.serializer.version
        + '">'
        + spriteStr
        + '</sprites>';	
	if (NetworkClient.io !== null && NetworkClient.io.connected)	{
		NetworkClient.io.emit("send_sprite",{"name":sprite.name, "data":spriteStr});
		this.inform("SUCCESS","sprite send");
	} else {
		this.inform("Error!", "Network not connected");
	}	
};

IDE_Morph.prototype.networkMenu = function () {
	var menu,
		world = this.world(),
        pos = this.controlBar.cloudButton.bottomLeft();
		
	menu = new MenuMorph(this);    
	if (NetworkClient.io == null || !NetworkClient.io.connected){
		menu.addItem('Connect to Network', 'netConnect');
	} else {
		menu.addItem('Disconnect', function(){
			NetworkClient.io.disconnect();
		});
	}
	
	menu.popup(world, pos);
};

IDE_Morph.prototype.netConnect = function(){
	var myself=this;
	new DialogBoxMorph(
        null,
        function (data) {
			NetworkClient.serverAddress = data.serverAddress || "localhost";
			NetworkClient.nickName = data.nickName || "unknown";
			NetworkClient.connect(myself);
		}).netConnectDialog(this.world());
}

//======= Network support ==========
Process.prototype.doBroadcastOnNetwork = function (message) {
	if (NetworkClient.io !== null && NetworkClient.io.connected){
		NetworkClient.io.send(message);
	}
}

Process.prototype.doReportUserData = function (varName,user) {	
	//helper, for converting a JSON object containing a list, to a List object
	//It goes deep, to support nested lists.
	//Generally, shared lists should be handled with care, as they could impact performance
	function listify(v){ 
		if (v.hasOwnProperty('contents')){
			for (var i=0; i<v.contents.length; i++){
				v.contents[i] = listify(v.contents[i]);
			}
			return new List(v.contents);
		} else {
			return v;
		}
		
	}
//user is an array: [0] nickname, [1] socketid
	var value;
	if (NetworkClient.users[user[1]] && NetworkClient.users[user[1]].variables[varName]){
		value = NetworkClient.users[user[1]].variables[varName];
		if (value.hasOwnProperty('contents')){ //if it is an object containing a list, convert it to a List object
			return listify(value);	
		} else {
			return value;
		}	
	}
	return '';
}

Process.prototype.doBroadcastVariable = function (variable,recipient){
// variable is an array, [0] : variable name, [1]: variableframe.vars object
// recipient is an array, [0]: nickname, [1]: socketid or all
	if (recipient===''){
		NetworkClient.io.emit("share_variable",{ "data": {"name":variable[0],"value":variable[1][variable[0]].value}});
	} else{
		NetworkClient.io.emit("share_variable",{ "data": {"name":variable[0],"value":variable[1][variable[0]].value}, "recipient": recipient[1]})
	}	
}
//======= Network support ==========